using System;
using System.Collections;
using System.Collections.Generic;
using System.Data;
using System.Text;
using Vevo.Domain;
using Vevo.Domain.DataInterfaces;
using Vevo.Domain.Users;
using Vevo.Shared.DataAccess;
using Vevo.Shared.Utilities;
using Vevo.Data.Implementation;
using Vevo.Base.Data;
using Vevo.Base.Domain;

namespace Vevo.Data
{
    public class CustomerRepositoryNoCache : ICustomerRepository
    {
        #region Private
        private ShippingAddressAccess _shippingAddressAccess = new ShippingAddressAccess();

        private const string AllCustomerFields =
            "CustomerID, RegisterDate, UserName, FirstName, LastName, Company, Address1, Address2, " +
            "City, State, Zip, Country, Phone, Fax, Email, UseBillingAsShipping,  " +
            "MerchantNotes, IsWholesale, WholesaleLevel, AffiliateCode, AffiliateExpireDate, IsTaxExempt, " +
            "IsEnabled, QBListID, IsQBExported, ProductIDsCompareList, TaxExemptID, TaxExemptCountry, TaxExemptState, FBUserID ";

        private Customer Create( Customer customer )
        {
            string newID = DataAccess.ExecuteScalar(
                "INSERT INTO Customer ( RegisterDate, UserName, FirstName, LastName, Company, " +
                "Address1, Address2, " +
                "City,  State, Zip, Country, Phone, Fax, Email, UseBillingAsShipping, " +
                "MerchantNotes, IsWholesale, WholesaleLevel, " +
                "AffiliateCode, AffiliateExpireDate, IsTaxExempt, IsEnabled, IsQBExported, ProductIDsCompareList, " +
                "TaxExemptID, TaxExemptCountry, TaxExemptState, FBUserID ) " +
                "VALUES ( @RegisterDate, @UserName, @FirstName, @LastName, @Company, " +
                "@Address1, @Address2, @City, " +
                "@State, @Zip, @Country, @Phone, @Fax, @Email, @UseBillingAsShipping, " +
                "@MerchantNotes, " +
                "@IsWholesale, @WholesaleLevel, @AffiliateCode, @AffiliateExpireDate, @IsTaxExempt, " +
                "@IsEnabled, @IsQBExported, @ProductIDsCompareList, @TaxExemptID, @TaxExemptCountry, @TaxExemptState, @FBUserID ); " +
                "SELECT @@Identity; ",
                DataAccess.CreateParameterDateTime( customer.RegisterDate ),
                DataAccess.CreateParameterString( customer.UserName ),
                DataAccess.CreateParameterString( customer.BillingAddress.FirstName ),
                DataAccess.CreateParameterString( customer.BillingAddress.LastName ),
                DataAccess.CreateParameterString( customer.BillingAddress.Company ),
                DataAccess.CreateParameterString( customer.BillingAddress.Address1 ),
                DataAccess.CreateParameterString( customer.BillingAddress.Address2 ),
                DataAccess.CreateParameterString( customer.BillingAddress.City ),
                DataAccess.CreateParameterString( customer.BillingAddress.State ),
                DataAccess.CreateParameterString( customer.BillingAddress.Zip ),
                DataAccess.CreateParameterString( customer.BillingAddress.Country ),
                DataAccess.CreateParameterString( customer.BillingAddress.Phone ),
                DataAccess.CreateParameterString( customer.BillingAddress.Fax ),
                DataAccess.CreateParameterString( customer.Email ),
                DataAccess.CreateParameterBool( customer.UseBillingAsShipping ),
                DataAccess.CreateParameterString( customer.MerchantNotes ),
                DataAccess.CreateParameterBool( customer.IsWholesale ),
                DataAccess.CreateParameterInt32( customer.WholesaleLevel ),
                DataAccess.CreateParameterString( customer.AffiliateCode ),
                DataAccess.CreateParameterDateTime( customer.AffiliateExpireDate ),
                DataAccess.CreateParameterBool( customer.IsTaxExempt ),
                DataAccess.CreateParameterBool( customer.IsEnabled ),
                DataAccess.CreateParameterBool( false ),
                DataAccess.CreateParameterString( customer.ProductIDsCompareList ),
                DataAccess.CreateParameterString( customer.TaxExemptID ),
                DataAccess.CreateParameterString( customer.TaxExemptCountry ),
                DataAccess.CreateParameterString( customer.TaxExemptState ),
                DataAccess.CreateParameterString( customer.FBUserID )
                );

            customer.CustomerID = newID;
            return customer;
        }

        private Customer Update( Customer customer )
        {
            DataAccess.ExecuteNonQuery(
                "UPDATE Customer " +
                "SET " +
                "    RegisterDate = @RegisterDate, " +
                "    UserName = @UserName, " +
                "    FirstName = @FirstName, LastName = @LastName, " +
                "    Company = @Company, Address1 = @Address1, " +
                "    Address2 = @Address2, City = @City, " +
                "    State = @State, Zip = @Zip, " +
                "    Country = @Country, Phone = @Phone, " +
                "    Fax = @Fax, Email = @Email, UseBillingAsShipping = @UseBillingAsShipping, " +
                "    MerchantNotes = @MerchantNotes, " +
                "    IsWholesale = @IsWholesale, " +
                "    WholesaleLevel = @WholesaleLevel, " +
                "    IsTaxExempt = @IsTaxExempt, " +
                "    IsEnabled = @IsEnabled, " +
                "    AffiliateCode = @AffiliateCode, " +
                "    AffiliateExpireDate = @AffiliateExpireDate, " +
                "    QBListID = @QBListID, " +
                "    IsQBExported = @IsQBExported, " +
                "    ProductIDsCompareList = @ProductIDsCompareList, " +
                "    TaxExemptID = @TaxExemptID, " +
                "    TaxExemptCountry = @TaxExemptCountry, " +
                "    TaxExemptState = @TaxExemptState, " +
                "    FBUserID = @FBUserID " +
                "    WHERE CustomerID = @CustomerID;",

                DataAccess.CreateParameterDateTime( customer.RegisterDate ),
                DataAccess.CreateParameterString( customer.UserName ),
                DataAccess.CreateParameterString( customer.BillingAddress.FirstName ),
                DataAccess.CreateParameterString( customer.BillingAddress.LastName ),
                DataAccess.CreateParameterString( customer.BillingAddress.Company ),
                DataAccess.CreateParameterString( customer.BillingAddress.Address1 ),
                DataAccess.CreateParameterString( customer.BillingAddress.Address2 ),
                DataAccess.CreateParameterString( customer.BillingAddress.City ),
                DataAccess.CreateParameterString( customer.BillingAddress.State ),
                DataAccess.CreateParameterString( customer.BillingAddress.Zip ),
                DataAccess.CreateParameterString( customer.BillingAddress.Country ),
                DataAccess.CreateParameterString( customer.BillingAddress.Phone ),
                DataAccess.CreateParameterString( customer.BillingAddress.Fax ),
                DataAccess.CreateParameterString( customer.Email ),
                DataAccess.CreateParameterBool( customer.UseBillingAsShipping ),
                DataAccess.CreateParameterString( customer.MerchantNotes ),
                DataAccess.CreateParameterBool( customer.IsWholesale ),
                DataAccess.CreateParameterInt32( customer.WholesaleLevel ),
                DataAccess.CreateParameterBool( customer.IsTaxExempt ),
                DataAccess.CreateParameterBool( customer.IsEnabled ),
                DataAccess.CreateParameterString( customer.AffiliateCode ),
                DataAccess.CreateParameterDateTime( customer.AffiliateExpireDate ),
                DataAccess.CreateParameterString( customer.QBListID ),
                DataAccess.CreateParameterBool( customer.IsQBExported ),
                DataAccess.CreateParameterString( customer.ProductIDsCompareList ),
                DataAccess.CreateParameterString( customer.TaxExemptID ),
                DataAccess.CreateParameterString( customer.TaxExemptCountry ),
                DataAccess.CreateParameterString( customer.TaxExemptState ),
                DataAccess.CreateParameterString( customer.FBUserID ),
                DataAccess.CreateParameterString( customer.CustomerID ) );

            return customer;
        }

        private Customer CreateOneFromRow( DataRow row )
        {
            Customer customer = new Customer();
            customer.CustomerID = row["CustomerID"].ToString();
            customer.RegisterDate = ConvertUtilities.ToDateTime( row["RegisterDate"] );
            customer.UserName = row["UserName"].ToString();
            customer.BillingAddress = new Address(
                row["FirstName"].ToString(), row["LastName"].ToString(), row["Company"].ToString(),
                row["Address1"].ToString(), row["Address2"].ToString(), row["City"].ToString(),
                row["State"].ToString(), row["Zip"].ToString(), row["Country"].ToString(),
                row["Phone"].ToString(), row["Fax"].ToString() );
            customer.Email = row["Email"].ToString();
            customer.UseBillingAsShipping = ConvertUtilities.ToBoolean( row["UseBillingAsShipping"] );

            customer.MerchantNotes = row["MerchantNotes"].ToString();
            customer.IsWholesale = ConvertUtilities.ToBoolean( row["IsWholesale"] );
            customer.WholesaleLevel = ConvertUtilities.ToInt32( row["WholesaleLevel"] );
            customer.AffiliateCode = row["AffiliateCode"].ToString();
            customer.AffiliateExpireDate = ConvertUtilities.ToDateTime( row["AffiliateExpireDate"] );
            customer.IsTaxExempt = ConvertUtilities.ToBoolean( row["IsTaxExempt"] );
            customer.IsEnabled = ConvertUtilities.ToBoolean( row["IsEnabled"] );
            customer.QBListID = row["QBListID"].ToString();
            customer.IsQBExported = ConvertUtilities.ToBoolean( row["IsQBExported"] );
            customer.ProductIDsCompareList = row["ProductIDsCompareList"].ToString();
            customer.TaxExemptID = row["TaxExemptID"].ToString();
            customer.TaxExemptCountry = row["TaxExemptCountry"].ToString();
            customer.TaxExemptState = row["TaxExemptState"].ToString();
            customer.FBUserID = row["FBUserID"].ToString();

            return customer;
        }

        private IList<Customer> ConvertTableToList( DataTable table )
        {
            return ConvertTableToList( table, 0, table.Rows.Count );
        }

        private IList<Customer> ConvertTableToList( DataTable table, int startIndex, int endIndex )
        {
            IList<Customer> list = new List<Customer>();

            for (int i = startIndex; i <= endIndex && i < table.Rows.Count; i++)
                list.Add( CreateOneFromRow( table.Rows[i] ) );

            return list;
        }

        private string CreateSortByText( string sortBy )
        {
            if (sortBy == null || sortBy.Trim() == "")
                return "";
            else
                return "ORDER BY " + sortBy;
        }

        private DataTable SearchCustomer(
          string sortBy,
          SearchFilter searchFilter,
          BoolFilter isWholesale )
        {
            ArrayList parameterList = new ArrayList();
            SearchFilterHelperNew searchFilterHelper = new SearchFilterHelperNew();
            string commandSearch = searchFilterHelper.CreateCommandWithWhere( searchFilter, searchFilter.FieldName, parameterList );

            DataTable table = DataAccess.ExecuteSelect(
                "SELECT " + AllCustomerFields +
                "FROM Customer " +
                commandSearch +
                FlagFilterHelper.CreateFlagFilterWithAnd( "isWholesale", RepositoryUtilities.ConvertToFlagFilter( isWholesale ) ) +
                CreateSortByText( sortBy ) + "; ",
                DataAccess.ConvertArrayListToDbParameters( parameterList )
                );

            return table;
        }

        private DataTable SearchCustomer(
          string sortBy,
          SearchFilter searchFilter,
          BoolFilter isWholesale,
          int startIndex,
          int endIndex )
        {
            ArrayList parameterList = new ArrayList();
            SearchFilterHelperNew searchFilterHelper = new SearchFilterHelperNew();
            string commandSearch = searchFilterHelper.CreateCommandWithWhere( searchFilter, searchFilter.FieldName, parameterList );

            string query = "SELECT ROW_NUMBER() over ( " + CreateSortByText( sortBy ) + ") as rownum, " +
                " * FROM (" +
                "SELECT " + AllCustomerFields +
                "FROM Customer " +
                commandSearch +
                FlagFilterHelper.CreateFlagFilterWithAnd( "isWholesale", RepositoryUtilities.ConvertToFlagFilter( isWholesale ) ) +
                ")AS Customer";

            string extQuery = "with SearchResult as (" + query + ") " +
                    " select * from SearchResult where rownum between " + (startIndex + 1).ToString() + " and " + (endIndex + 1).ToString() + ";";

            DataTable table = DataAccess.ExecuteSelect( extQuery,
                DataAccess.ConvertArrayListToDbParameters( parameterList )
                );

            return table;
        }

        private int GetCountSearchCustomer(
          string sortBy,
          SearchFilter searchFilter,
          BoolFilter isWholesale )
        {
            ArrayList parameterList = new ArrayList();
            SearchFilterHelperNew searchFilterHelper = new SearchFilterHelperNew();
            string commandSearch = searchFilterHelper.CreateCommandWithWhere( searchFilter, searchFilter.FieldName, parameterList );

            string query = "SELECT COUNT(*) FROM (" +
                "SELECT " + AllCustomerFields +
                "FROM Customer " +
                commandSearch +
                FlagFilterHelper.CreateFlagFilterWithAnd( "isWholesale", RepositoryUtilities.ConvertToFlagFilter( isWholesale ) ) +
                ") AS Row";

            string countResult = DataAccess.ExecuteScalar( query,
                DataAccess.ConvertArrayListToDbParameters( parameterList ) );

            return Convert.ToInt32( countResult );
        }

        #endregion


        #region Aggregate Methods

        #endregion

        #region Public Methods

        public Customer Save( Customer customer )
        {
            Customer result;

            if (customer.CustomerID == "0")
                result = Create( customer );
            else
                result = Update( customer );

            _shippingAddressAccess.SaveAll( customer.CustomerID, customer.ShippingAddresses );

            return result;
        }

        public Customer GetOne( string customerID )
        {
            DataTable table = DataAccess.ExecuteSelect(
                "SELECT " + AllCustomerFields +
                "FROM Customer " +
                "WHERE CustomerID = @CustomerID; ",
                DataAccess.CreateParameterString( customerID ) );

            if (table.Rows.Count > 0)
                return CreateOneFromRow( table.Rows[0] );
            else
                return Customer.Null;
        }

        public string GetIDFromUserName( string userName )
        {
            DataTable table = DataAccess.ExecuteSelect(
                "SELECT " + AllCustomerFields +
                "FROM Customer " +
                "WHERE UserName = @UserName; ",
                DataAccess.CreateParameterString( userName ) );

            if (table.Rows.Count > 0)
                return table.Rows[0]["CustomerID"].ToString();
            else
                return "0";

        }

        public void Delete( string customerID )
        {
            Customer customer = GetOne( customerID );
            customer.ShippingAddresses.Clear();

            using (DataTransactionScope transaction = new DataTransactionScope())
            {
                DataAccess.ExecuteNonQuery(
                 "DELETE " +
                 "FROM Customer " +
                 "WHERE CustomerID = @CustomerID; ",
                 DataAccess.CreateParameterString( customerID ) );

                _shippingAddressAccess.SaveAll( customer.CustomerID, customer.ShippingAddresses );
                transaction.Complete();
            }
        }

        public IList<Customer> SearchCustomer(
            string sortBy,
            SearchFilter searchFilter,
            BoolFilter isWholesale,
            int startIndex,
            int endIndex,
            out int howManyItems )
        {
            DataTable table = SearchCustomer( sortBy, searchFilter, isWholesale, startIndex, endIndex );

            howManyItems = GetCountSearchCustomer( sortBy, searchFilter, isWholesale );

            //DataColumn[] keys = new DataColumn[] { table.Columns["CustomerID"] };
            //table.PrimaryKey = keys;
            return ConvertTableToList( table );
        }

        public IList<Customer> ExportCustomer(
            string sortBy,
            SearchFilter searchFilter,
            BoolFilter isWholesale )
        {
            return ConvertTableToList( SearchCustomer( sortBy, searchFilter, isWholesale ) );
        }


        public IList<Customer> GetCustomerByEmail( string email )
        {
            DataTable table = DataAccess.ExecuteSelect(
                "SELECT " + AllCustomerFields +
                "FROM Customer " +
                "WHERE Email = @Email; ",
                DataAccess.CreateParameterString( email )
                );

            DataColumn[] keys = new DataColumn[] { table.Columns["CustomerID"] };
            table.PrimaryKey = keys;
            return ConvertTableToList( table );
        }

        public IList<TableSchemaItem> GetTableSchema()
        {
            DataTable table = DataAccess.ExecuteSelect(
                "SELECT " + AllCustomerFields +
                "FROM Customer " +
                "WHERE 0 = 1; " );

            return RepositoryUtilities.ConvertTableToSchemaItemList( table );
        }

        public void UpdateWholesalesLevel( int wholesaleLevel )
        {
            DataAccess.ExecuteNonQuery(
               "UPDATE Customer " +
               "SET WholesaleLevel = @WholesaleLevel " +
               "WHERE WholesaleLevel > @WholesaleLevel1 AND IsWholesale = @IsWholesale ;",
               DataAccess.CreateParameterInt32( wholesaleLevel ),
               DataAccess.CreateParameterInt32( wholesaleLevel ),
               DataAccess.CreateParameterBool( true )
               );
        }

        public IList<Customer> GetAll( string sortBy )
        {
            DataTable table = DataAccess.ExecuteSelect(
                "SELECT " + AllCustomerFields +
                "FROM Customer " +
                CreateSortByText( sortBy ) + "; " );
            return ConvertTableToList( table );
        }

        public IList<Customer> GetLastestCustomers(int limitCount)
        {
            DataTable table = DataAccess.ExecuteSelect(
                "SELECT TOP " + limitCount.ToString() + " " + AllCustomerFields +
                " FROM Customer ORDER BY RegisterDate DESC"
                );
            return ConvertTableToList( table );
        }

        public IList<Customer> GetByIsQBExported( bool isQBExported, string existList )
        {
            string query = "SELECT " + AllCustomerFields +
                "FROM Customer " +
                "WHERE IsQBExported = @IsQBExported";

            if (!string.IsNullOrEmpty( existList ))
            {
                query += " AND QBListID NOT IN ( " + existList + "  ); ";
            }
            else
            {
                query += "; ";
            }

            DataTable table = DataAccess.ExecuteSelect( query, DataAccess.CreateParameterBool( isQBExported ) );
            return ConvertTableToList( table );
        }

        public void SetDefaultQBData( string qbListID )
        {
            DataAccess.ExecuteNonQuery(
                "UPDATE Customer " +
                "SET QBListID = '', " +
                "IsQBExported = @IsQBExported " +
                "WHERE QBListID = @QBListID;",
                DataAccess.CreateParameterBool( false ),
                DataAccess.CreateParameterString( qbListID )
            );
        }

        public Customer GetOneByFBUserID( string FBUserID )
        {
            DataTable table = DataAccess.ExecuteSelect(
                "SELECT " + AllCustomerFields +
                "FROM Customer " +
                "WHERE FBUserID = @FBUserID; ",
                DataAccess.CreateParameterString( FBUserID ) );

            if (table.Rows.Count > 0)
                return CreateOneFromRow( table.Rows[0] );
            else
                return Customer.Null;
        }

        public IList<ShippingAddress> GetAllShippingAddress(string sortBy)
        {
            return _shippingAddressAccess.GetAll( sortBy );
        }

        public IList<ShippingAddress> GetShippingAddressByCustomerID( string customerID )
        {
            return _shippingAddressAccess.GetByCustomerID( customerID );
        }

        public IList<TableSchemaItem> GetShippingAddressTableSchemas()
        {
            return _shippingAddressAccess.GetTableSchemas();
        }

        public IList<ShippingAddress> SearchShippingAddress(
            string customerID,
            string sortBy,
            SearchFilter searchFilter,
            int startIndex,
            int endIndex,
            out int howManyItems )
        {
            return _shippingAddressAccess.SearchShippingAddress( customerID, sortBy, searchFilter, startIndex, endIndex, out howManyItems );
        }
        public void ClearCustomerCache()
        { 
            
        }

        public void DeleteAllShippingAddress()
        {
            _shippingAddressAccess.DeleteAllShippingAddress();
        }

        #endregion
    }
}
