using System;
using System.Collections.Generic;
using System.Text;

using Vevo.Domain;
using Vevo.Domain.Shipping;
using Vevo.Domain.DataInterfaces;
using System.Data;
using Vevo.Shared.DataAccess;
using System.Collections;
using Vevo.Base.Data.Implementation;
using Vevo.Base.Data;
using Vevo.Base.Domain;

namespace Vevo.Data.Implementation
{
    internal class ShippingZoneItemAccess : BaseListedDataAccess<string, ShippingZoneItem>
    {
        #region Private

        private const string AllShippingZoneItemField = "ZoneItemID, ZoneGroupID, CountryCode, StateCode, ZipCode ";

        private ShippingZoneItem CreateOneShippingZoneItemFromRow( DataRow row )
        {
            ShippingZoneItem zoneItem = new ShippingZoneItem();
            zoneItem.ZoneItemID = row["ZoneItemID"].ToString();
            zoneItem.ZoneGroupID = row["ZoneGroupID"].ToString();
            zoneItem.CountryCode = row["CountryCode"].ToString();
            zoneItem.StateCode = row["StateCode"].ToString();
            zoneItem.ZipCode = row["ZipCode"].ToString();

            return zoneItem;
        }

        private IList<ShippingZoneItem> ConvertShippingZoneItemTableToList( DataTable table )
        {
            return ConvertShippingZoneItemTableToList( table, 0, table.Rows.Count );
        }

        private IList<ShippingZoneItem> ConvertShippingZoneItemTableToList( DataTable table, int startIndex, int endIndex )
        {
            IList<ShippingZoneItem> list = new List<ShippingZoneItem>();

            foreach (DataRow row in table.Rows)
            {
                list.Add( CreateOneShippingZoneItemFromRow( row ) );
            }

            return list;
        }

        private string CreateSortByText( string sortBy )
        {
            if (sortBy == null || sortBy.Trim() == "")
                return "";
            else
                return "ORDER BY " + sortBy;
        }

        #endregion



        #region Protected

        protected override void Create( string zoneGroupID, ShippingZoneItem zoneItem )
        {
            zoneItem.ZoneItemID = DataAccess.ExecuteScalar(
                 "INSERT INTO ShippingZoneItem ( ZoneGroupID, CountryCode, StateCode, ZipCode ) " +
                 "VALUES (@ZoneGroupID, @CountryCode, @StateCode, @ZipCode ); " +
                 "SELECT @@Identity",
                 DataAccess.CreateParameterString( zoneGroupID ),
                 DataAccess.CreateParameterString( zoneItem.CountryCode ),
                 DataAccess.CreateParameterString( zoneItem.StateCode ),
                 DataAccess.CreateParameterString( zoneItem.ZipCode )
             );

            zoneItem.ZoneGroupID = zoneGroupID;
        }

        protected override void Update( string zoneGroupID, ShippingZoneItem zoneItem )
        {
            DataAccess.ExecuteSelect(
                 "UPDATE ShippingZoneItem " +
                 "SET " +
                 "   ZoneGroupID = @ZoneGroupID, " +
                 "   CountryCode = @CountryCode, " +
                 "   StateCode = @StateCode, " +
                 "   ZipCode = @ZipCode " +
                 "WHERE ZoneItemID  = @ZoneItemID; ",
                 DataAccess.CreateParameterString( zoneItem.ZoneGroupID ),
                 DataAccess.CreateParameterString( zoneItem.CountryCode ),
                 DataAccess.CreateParameterString( zoneItem.StateCode ),
                 DataAccess.CreateParameterString( zoneItem.ZipCode ),
                 DataAccess.CreateParameterString( zoneItem.ZoneItemID )
              );
        }

        protected override void Delete( string zoneGroupID, ShippingZoneItem zoneItem )
        {
            DataAccess.ExecuteNonQuery(
                "DELETE FROM ShippingZoneItem " +
                "WHERE ZoneItemID = @DZoneItemID;",
                DataAccess.CreateParameterString( zoneItem.ZoneItemID ) );
        }

        #endregion


        #region Public Methods

        public IList<ShippingZoneItem> GetShippingZoneItemByZoneGroupID( string zoneGroupID )
        {
            DataTable table = DataAccess.ExecuteSelect(
                "SELECT " + AllShippingZoneItemField + " FROM ShippingZoneItem " +
                "WHERE ShippingZoneItem.ZoneGroupID = @ZoneGroupID " +
                "ORDER BY ZoneItemID; ",
                DataAccess.CreateParameterString( zoneGroupID ) );

            return ConvertShippingZoneItemTableToList( table );
        }

        public IList<ShippingZoneItem> SearchShippingZoneItems(
            string zoneGroupID,
            string sortBy,
            SearchFilter searchFilter,
            int startIndex,
            int endIndex,
            out int howManyItems )
        {
            ArrayList parameterList = new ArrayList();
            SearchFilterHelperNew searchFilterHelper = new SearchFilterHelperNew();
            string commandSearch = searchFilterHelper.CreateCommandWithWhere( searchFilter, searchFilter.FieldName, parameterList );
            parameterList.Add( DataAccess.CreateParameterString( zoneGroupID ) );

            DataTable table = DataAccess.ExecuteSelect(
                "SELECT " + AllShippingZoneItemField +
                "FROM ShippingZoneItem " + commandSearch + " AND ZoneGroupID = @ZoneGroupID " +
                CreateSortByText( sortBy ) + "; ",
                DataAccess.ConvertArrayListToDbParameters( parameterList ) );

            howManyItems = table.Rows.Count;

            DataColumn[] keys = new DataColumn[] { table.Columns["ZoneItemID"] };
            table.PrimaryKey = keys;

            return ConvertShippingZoneItemTableToList( table, startIndex, endIndex );
        }

        public IList<TableSchemaItem> GetShippingZoneItemTableSchemas()
        {
            DataTable table = DataAccess.ExecuteSelect(
                "SELECT " + AllShippingZoneItemField +
                "FROM ShippingZoneItem " +
                "WHERE 0 = 1; " );

            return RepositoryUtilities.ConvertTableToSchemaItemList( table );
        }

        #endregion

    }
}
