using System.Collections;
using System.Collections.Generic;
using System.Data;
using System.Web.UI;
using Vevo.Base.Data.Caching;
using Vevo.Domain;
using Vevo.Domain.Base;
using Vevo.Domain.DataInterfaces;
using Vevo.Domain.Products;
using Vevo.Shared.Utilities;
using System;
using Vevo.Base.Domain;

namespace Vevo.Data
{
    public class ProductRepository : IProductRepository
    {
        #region Private

        private const string ProductRepositoryName = "ProductRepository";
        private const string OptionGroupRepositoryName = "OptionGroupRepository";
        private const string GoogleFeedMappingRepositoryName = "GoogleFeedMappingRepository";
        private const string SpecificationItemRepositoryName = "SpecificationItemRepository";

        private const string GetOneSignature = "GetOne";
        private const string GetProductIDBySkuSignature = "GetProductIDBySku";
        private const string GetProductIDAndSkuSignature = "GetProductIDAndSku";
        private const string GetProductImagesSignature = "GetProductImages";
        private const string GetAllProductCountSignature = "GetAllProductCount";
        private const string GetAllProductCountBySkuSignature = "GetAllProductCountBySku";
        private const string GetAllProductInactiveCountSignature = "GetAllProductInactiveCount";
        private const string GetAllProductInLowStockCountSignature = "GetAllProductInLowStockCount";
        private const string GetProductCountByCategoryIDSignature = "GetProductCountByCategoryID";
        private const string GetProductCountByDepartmentIDSignature = "GetProductCountByDepartmentID";
        private const string GetProductCountByManufacturerIDSignature = "GetProductCountByManufacturerID";
        private const string GetProductCountGroupByCategoryIDSignature = "GetProductCountGroupByCategoryID";
        private const string GetProductCountGroupByDepartmentIDSignature = "GetProductCountGroupByDepartmentID";
        private const string GetFacetCountSignature = "GetFacetCount";
        private const string GetMaxPriceByCategoryDepartmentSignature = "GetMaxPriceByCategoryDepartment";
        private const string GetAllByBestSellingSignature = "GetAllByBestSelling";
        private const string GetAllByRandomSignature = "GetAllByRandom";
        private const string GetAllByNewArrivalSignature = "GetAllByNewArrival";
        private const string GetAllByTodaySpecialSignature = "GetAllByTodaySpecial";
        private const string GetAllByTaxClassIDSignature = "GetAllByTaxClassID";
        private const string GetTableSchemaSignature = "GetTableSchema";
        private const string GetProductOptionGroupsSignature = "GetProductOptionGroups";
        private const string GetProductStocksSignature = "GetProductStocks";
        private const string GetProductShippingCostsSignature = "GetProductShippingCosts";
        private const string GetProductPricesSignature = "GetProductPrices";
        private const string GetDefaultProductPriceListSignature = "GetDefaultProductPriceList";
        private const string GetDefaultProductPriceSignature = "GetDefaultProductPrice";
        private const string DeleteProductPricesByStoreIDSignature = "DeleteProductPricesByStoreID";
        private const string CreateProductPricesFromDefaultByStoreIDSignature = "CreateProductPricesFromDefaultByStoreID";
        private const string GetProductIDFromUrlNameSignature = "GetProductIDFromUrlName";
        private const string GetLocalesSignature = "GetLocales";
        private const string GetImageLocalesSignature = "GetImageLocales";
        private const string GetMetaInformationsByStoreIDSignature = "GetMetaInformationsByStoreID";
        private const string GetMetaInformationsSignature = "GetMetaInformations";
        private const string GetByCategoryIDSignature = "GetByCategoryID";
        private const string GetNewArrivalByCategoryDepartmentIDSignature = "GetNewArrivalByCategoryDepartmentID";
        private const string GetNewArrivalByManufacturerIDSignature = "GetNewArrivalByManufacturerID";
        private const string GetFacetResultByCategoryIDSignature = "GetFacetResultByCategoryID";
        private const string GetByDepartmentIDSignature = "GetByDepartmentID";
        private const string GetByManufacturerIDSignature = "GetByManufacturerID";
        private const string ImageIsUsedByAnotherProductSignature = "ImageIsUsedByAnotherProduct";
        private const string IsRegularImageExistingSignature = "IsRegularImageExisting";
        private const string IsLargeImageExistingSignature = "IsLargeImageExisting";
        private const string IsThumbnailImageExistingSignature = "IsThumbnailImageExisting";
        private const string IsImageSecondaryExistingSignature = "IsImageSecondaryExisting";
        private const string HasProductInDiscountGroupSignature = "HasProductInDiscountGroup";
        private const string SearchProductSignature = "SearchProduct";
        private const string QuickSearchSignature = "QuickSearch";
        private const string AdvancedSearchSignature = "AdvancedSearch";
        private const string GetStockOptionLineSignature = "GetStockOptionLine";
        private const string CountQuickSearchResultSignature = "CountQuickSearchResult";
        private const string GetProductSpecificationsSignature = "GetProductSpecifications";
        private const string GetProductSpecificationsIDByProductIDsSignature = "GetProductSpecificationsIDByProductIDs";
        private const string GetAllBySubscriptionLevelIDSignature = "GetAllBySubscriptionLevelID";
        private const string GetProductKitsSignature = "GetProductKits";
        private const string GetAllProductKitSignature = "GetAllProductKit";
        private const string GetProductKitItemsSignature = "GetProductKitItems";
        private const string GetAllProductSpecificationsSignature = "GetAllProductSpecifications";
        private const string CheckIsProductKitByProductIDSignature = "CheckIsProductKitByProductID";
        private const string GetMultipleCategoryProductSignature = "GetMultipleCategoryProduct";
        private const string GetMultipleDepartmentProductSignature = "GetMultipleDepartmentProduct";
        private const string GetProductIDListByCategoryDepartmentSignature = "GetProductIDListByCategoryDepartment";
        private const string GetProductNewArrivalSignature = "GetProductNewArrival";

        private DataCache _dataCache_Product = new DataCache( ProductRepositoryName );
        private DataCache _dataCache_OptionGroup = new DataCache( OptionGroupRepositoryName );
        private DataCache _dataCache_GoogleFeedMapping = new DataCache( GoogleFeedMappingRepositoryName );
        private DataCache _dataCache_SpecificationItem = new DataCache( SpecificationItemRepositoryName );

        private ProductRepositoryNoCache _repository = new ProductRepositoryNoCache();

        private ICacheDependencyUtility _cacheDependency = new CacheDependencyUtility( new string[] { 
            "OptionItem", 
            "OptionItemLocale", 
            "Product", 
            "ProductStock", 
            "ProductLocale", 
            "ProductPrice",
            "ProductRecurring", 
            "ProductCategory",
            "Category",
            "ProductMetaInformation",
            "ProductCustomPrice",
            "ProductSpecification",
            "PromotionProduct",
			"ProductSubscription",
            "ProductKit"
        } );

        #endregion

        #region Constructors

        public ProductRepository()
        {
        }

        public ProductRepository( ICacheDependencyUtility cacheDependency )
        {
            _cacheDependency = cacheDependency;
        }

        #endregion

        #region Aggregate Members
        public IEntityList<ProductImage> GetProductImages( string productID )
        {
            IEntityList<ProductImage> productImages =
                (IEntityList<ProductImage>) _dataCache_Product.GetValue(
                GetProductImagesSignature, _cacheDependency,
                productID );

            if (productImages == null)
            {
                productImages = _repository.GetProductImages( productID );
                _dataCache_Product.SetValue( GetProductImagesSignature, productImages, productID );
            }

            return productImages;
        }

        public IList<ProductOptionGroup> GetProductOptionGroups( Culture culture, string productID )
        {
            IList<ProductOptionGroup> productOptionGroups =
                (IList<ProductOptionGroup>) _dataCache_Product.GetValue(
                    GetProductOptionGroupsSignature,
                    _cacheDependency,
                    culture.CultureID,
                    productID );

            if (productOptionGroups == null)
            {
                productOptionGroups = _repository.GetProductOptionGroups( culture, productID );
                _dataCache_Product.SetValue( GetProductOptionGroupsSignature, productOptionGroups, culture.CultureID, productID );
            }

            return productOptionGroups;
        }

        public IList<ProductStock> GetProductStocks( string productID )
        {
            IList<ProductStock> productStocks =
                (IList<ProductStock>) _dataCache_Product.GetValue(
                    GetProductStocksSignature,
                    _cacheDependency,
                    productID );

            if (productStocks == null)
            {
                productStocks = _repository.GetProductStocks( productID );
                _dataCache_Product.SetValue( GetProductStocksSignature, productStocks, productID );
            }

            return productStocks;
        }

        public IList<ProductShippingCost> GetProductShippingCosts( string productID )
        {
            IList<ProductShippingCost> list =
                (IList<ProductShippingCost>) _dataCache_Product.GetValue(
                    GetProductShippingCostsSignature,
                    _cacheDependency,
                    productID );

            if (list == null)
            {
                list = _repository.GetProductShippingCosts( productID );
                _dataCache_Product.SetValue( GetProductShippingCostsSignature, list, productID );
            }

            return list;
        }

        public IList<ProductSpecification> GetProductSpecifications( string productID )
        {
            IList<ProductSpecification> list =
                (IList<ProductSpecification>) _dataCache_Product.GetValue(
                    GetProductSpecificationsSignature,
                    _cacheDependency,
                    productID );

            if (list == null)
            {
                list = _repository.GetProductSpecifications( productID );
                _dataCache_Product.SetValue( GetProductSpecificationsSignature, list, productID );
            }

            return list;
        }

        public IList<string> GetProductSpecificationsIDByProductIDs( IList<String> productIDs )
        {
            IList<string> list =
                (IList<string>) _dataCache_Product.GetValue(
                GetProductSpecificationsIDByProductIDsSignature,
                _cacheDependency,
                productIDs );

            if (list == null)
            {
                list = _repository.GetProductSpecificationsIDByProductIDs( productIDs );
                _dataCache_Product.SetValue( GetProductSpecificationsIDByProductIDsSignature, list, productIDs );
            }

            return list;
        }

        public IList<ILocale> GetLocales( string productID )
        {
            IList<ILocale> list = (IList<ILocale>) _dataCache_Product.GetValue( GetLocalesSignature, _cacheDependency, productID );

            if (list == null)
            {
                list = _repository.GetLocales( productID );
                _dataCache_Product.SetValue( GetLocalesSignature, list, productID );
            }

            return list;
        }

        public IList<ILocale> GetImageLocales( string productImageID )
        {
            IList<ILocale> list = (IList<ILocale>) _dataCache_Product.GetValue( GetImageLocalesSignature, _cacheDependency, productImageID );

            if (list == null)
            {
                list = _repository.GetImageLocales( productImageID );
                _dataCache_Product.SetValue( GetImageLocalesSignature, list, productImageID );
            }

            return list;
        }

        public IList<ILocale> GetMetaInformationsByStoreID( string productID, string storeID )
        {
            IList<ILocale> list = (IList<ILocale>) _dataCache_Product.GetValue( GetMetaInformationsByStoreIDSignature, _cacheDependency, productID, storeID );

            if (list == null)
            {
                list = _repository.GetMetaInformationsByStoreID( productID, storeID );
                _dataCache_Product.SetValue( GetMetaInformationsByStoreIDSignature, list, productID, storeID );
            }

            return list;
        }

        public IList<ILocale> GetMetaInformations( string productID )
        {
            IList<ILocale> list = (IList<ILocale>) _dataCache_Product.GetValue( GetMetaInformationsSignature, _cacheDependency, productID );

            if (list == null)
            {
                list = _repository.GetMetaInformations( productID );
                _dataCache_Product.SetValue( GetMetaInformationsSignature, list, productID );
            }

            return list;
        }

        public void DeleteProductSpecification()
        {
            _repository.DeleteProductSpecification();
            _dataCache_Product.ClearRepositoryCache();
            _dataCache_OptionGroup.ClearRepositoryCache();
            _dataCache_GoogleFeedMapping.ClearRepositoryCache();
            _dataCache_SpecificationItem.ClearRepositoryCache();
        }

        public IList<ProductKit> GetProductKits( Culture culture, string productID )
        {
            IList<ProductKit> list =
                (IList<ProductKit>) _dataCache_Product.GetValue(
                    GetProductKitsSignature,
                    _cacheDependency,
                    culture.CultureID,
                    productID );

            if (list == null)
            {
                list = _repository.GetProductKits( culture, productID );
                _dataCache_Product.SetValue( GetProductKitsSignature, list, culture.CultureID, productID );
            }

            return list;
        }

        #endregion


        #region Public Members

        public Product Save( Product product )
        {
            // Need to write to database first before clearing cache.
            // Otherwise, cache may remember incorrect values before saving.
            Product newProduct = _repository.Save( product );
            _dataCache_Product.ClearRepositoryCache();
            _dataCache_OptionGroup.ClearRepositoryCache();
            _dataCache_GoogleFeedMapping.ClearRepositoryCache();
            _dataCache_SpecificationItem.ClearRepositoryCache();
            return newProduct;
        }

        public void Delete( string productID )
        {
            _repository.Delete( productID );
            _dataCache_Product.ClearRepositoryCache();
            _dataCache_OptionGroup.ClearRepositoryCache();
            _dataCache_GoogleFeedMapping.ClearRepositoryCache();
            _dataCache_SpecificationItem.ClearRepositoryCache();
        }

        public void DeleteLocalesByCultureID( string cultureID )
        {
            _repository.DeleteLocalesByCultureID( cultureID );
            _dataCache_Product.ClearRepositoryCache();
            _dataCache_OptionGroup.ClearRepositoryCache();
            _dataCache_GoogleFeedMapping.ClearRepositoryCache();
            _dataCache_SpecificationItem.ClearRepositoryCache();
        }

        public void DeleteAll()
        {
            _repository.DeleteAll();
            _dataCache_Product.ClearRepositoryCache();
            _dataCache_OptionGroup.ClearRepositoryCache();
            _dataCache_GoogleFeedMapping.ClearRepositoryCache();
            _dataCache_SpecificationItem.ClearRepositoryCache();
        }

        public void DeleteProductMetaInformationByStoreID( string storeID )
        {
            _repository.DeleteProductMetaInformationByStoreID( storeID );
            _dataCache_Product.ClearRepositoryCache();
            _dataCache_OptionGroup.ClearRepositoryCache();
            _dataCache_GoogleFeedMapping.ClearRepositoryCache();
            _dataCache_SpecificationItem.ClearRepositoryCache();
        }

        public void RefreshProductUrlNameAll( string productName )
        {
            _repository.RefreshProductUrlNameAll( productName );
            _dataCache_Product.ClearRepositoryCache();
            _dataCache_OptionGroup.ClearRepositoryCache();
            _dataCache_GoogleFeedMapping.ClearRepositoryCache();
            _dataCache_SpecificationItem.ClearRepositoryCache();

        }

        public void RefreshProductUrlNameAll()
        {
            _repository.RefreshProductUrlNameAll();
            _dataCache_Product.ClearRepositoryCache();
            _dataCache_OptionGroup.ClearRepositoryCache();
            _dataCache_GoogleFeedMapping.ClearRepositoryCache();
            _dataCache_SpecificationItem.ClearRepositoryCache();

        }

        public void UpdateSortOrder( string categoryID, string[] sortedProductIDs )
        {
            _repository.UpdateSortOrder( categoryID, sortedProductIDs );
            _dataCache_Product.ClearRepositoryCache();
            _dataCache_OptionGroup.ClearRepositoryCache();
            _dataCache_GoogleFeedMapping.ClearRepositoryCache();
            _dataCache_SpecificationItem.ClearRepositoryCache();
        }

        public void UpdateSortOrderDepartment( string departmentID, string[] sortedProductIDs )
        {
            _repository.UpdateSortOrderDepartment( departmentID, sortedProductIDs );
            _dataCache_Product.ClearRepositoryCache();
            _dataCache_OptionGroup.ClearRepositoryCache();
            _dataCache_GoogleFeedMapping.ClearRepositoryCache();
            _dataCache_SpecificationItem.ClearRepositoryCache();
        }

        public string GetProductIDBySku( string sku )
        {
            object productID = _dataCache_Product.GetValue( GetProductIDBySkuSignature, _cacheDependency, sku );
            if (productID == null)
            {
                productID = _repository.GetProductIDBySku( sku );
                _dataCache_Product.SetValue( GetProductIDBySkuSignature, productID, sku );
            }
            return productID.ToString();
        }

        public Dictionary<string, string> GetProductIDAndSku()
        {
            object skuProductID = _dataCache_Product.GetValue( GetProductIDAndSkuSignature, _cacheDependency );
            if (skuProductID == null)
            {
                skuProductID = _repository.GetProductIDAndSku();
                _dataCache_Product.SetValue( GetProductIDAndSkuSignature, skuProductID );
            }
            return (Dictionary<string, string>) skuProductID;
        }

        public Product GetOne( Culture culture, string productID, string storeID )
        {
            Product product = (Product) _dataCache_Product.GetValue(
                GetOneSignature,
                _cacheDependency,
                culture.CultureID,
                productID,
                storeID
                );

            if (product == null)
            {
                product = _repository.GetOne( culture, productID, storeID );
                _dataCache_Product.SetValue( GetOneSignature, product, culture.CultureID, productID, storeID );
            }
            return product;
        }

        public string GetProductIDFromUrlName( string urlName )
        {
            object productID = _dataCache_Product.GetValue( GetProductIDFromUrlNameSignature, _cacheDependency, urlName );

            if (productID == null)
            {
                productID = _repository.GetProductIDFromUrlName( urlName );
                _dataCache_Product.SetValue( GetProductIDFromUrlNameSignature, productID, urlName );
            }

            return productID.ToString();
        }

        public void RefreshIsParentVisibleByCategoryID( string categoryID, string storeID )
        {
            _repository.RefreshIsParentVisibleByCategoryID( categoryID, storeID );
            _dataCache_Product.ClearRepositoryCache();
            _dataCache_OptionGroup.ClearRepositoryCache();
            _dataCache_GoogleFeedMapping.ClearRepositoryCache();
            _dataCache_SpecificationItem.ClearRepositoryCache();
        }

        public void RefreshIsParentVisibleByDepartmentID( string departmentID, string storeID )
        {
            _repository.RefreshIsParentVisibleByDepartmentID( departmentID, storeID );
            _dataCache_Product.ClearRepositoryCache();
            _dataCache_OptionGroup.ClearRepositoryCache();
            _dataCache_GoogleFeedMapping.ClearRepositoryCache();
            _dataCache_SpecificationItem.ClearRepositoryCache();
        }

        public IList<Product> GetByCategoryID(
            Culture culture,
            string categoryID,
            string sortBy,
            BoolFilter isEnabled,
            string storeID
            )
        {
            IList<Product> list =
                (IList<Product>) _dataCache_Product.GetValue(
                    GetByCategoryIDSignature,
                    _cacheDependency,
                    culture.CultureID,
                    categoryID,
                    isEnabled,
                    sortBy,
                    storeID
                    );

            if (list == null)
            {
                list = _repository.GetByCategoryID(
                    culture,
                    categoryID,
                    sortBy,
                    isEnabled,
                    storeID
                    );

                _dataCache_Product.SetValue( GetByCategoryIDSignature, list, culture.CultureID, categoryID, isEnabled, sortBy, storeID );
            }

            return list;
        }

        public IList<Product> GetByCategoryID(
            Culture culture,
            string categoryID,
            string sortBy,
            BoolFilter isEnabled,
            bool isAffiliate,
            string storeID
        )
        {
            IList<Product> list =
                (IList<Product>) _dataCache_Product.GetValue(
                    GetByCategoryIDSignature,
                    _cacheDependency,
                    culture.CultureID,
                    categoryID,
                    isEnabled,
                    isAffiliate,
                    sortBy,
                    storeID
                    );

            if (list == null)
            {
                list = _repository.GetByCategoryID(
                    culture,
                    categoryID,
                    sortBy,
                    isEnabled,
                    isAffiliate,
                    storeID
                    );

                _dataCache_Product.SetValue( GetByCategoryIDSignature, list, culture.CultureID, categoryID, isEnabled, isAffiliate, sortBy, storeID );
            }

            return list;
        }

        public IList<Product> GetByDepartmentID(
            Culture culture,
            string departmentID,
            string sortBy,
            BoolFilter isEnabled,
           string storeID
        )
        {
            IList<Product> list =
                (IList<Product>) _dataCache_Product.GetValue(
                    GetByDepartmentIDSignature,
                    _cacheDependency,
                    culture.CultureID,
                    departmentID,
                    isEnabled,
                    sortBy,
                    storeID
                    );

            if (list == null)
            {
                list = _repository.GetByDepartmentID(
                    culture,
                    departmentID,
                    sortBy,
                    isEnabled,
                    storeID
                    );

                _dataCache_Product.SetValue( GetByDepartmentIDSignature, list, culture.CultureID, departmentID, isEnabled, sortBy, storeID );
            }

            return list;
        }

        public IList<Product> GetByCategoryID(
            Culture culture,
            string categoryID,
            string sortBy,
            int startIndex,
            int endIndex,
            BoolFilter isEnabled,
            bool isIncludeProductKit,
            out int howmanyItems,
            string storeID
            )
        {
            howmanyItems = 0;
            Pair list = (Pair) _dataCache_Product.GetValue(
                GetByCategoryIDSignature,
                _cacheDependency,
                culture.CultureID,
                categoryID,
                sortBy,
                startIndex,
                endIndex,
                isEnabled,
                isIncludeProductKit,
                storeID
                );
            if (list == null)
            {
                list = new Pair();
                list.First = _repository.GetByCategoryID(
                    culture,
                    categoryID,
                    sortBy,
                    startIndex,
                    endIndex,
                    isEnabled,
                    isIncludeProductKit,
                    out howmanyItems,
                    storeID
                    );

                list.Second = howmanyItems;
                _dataCache_Product.SetValue( GetByCategoryIDSignature, list, culture.CultureID, categoryID, sortBy, startIndex, endIndex, isEnabled, isIncludeProductKit, storeID );
            }
            howmanyItems = ConvertUtilities.ToInt32( list.Second );

            return (IList<Product>) list.First;
        }

        public IList<Product> GetByCategoryID(
            Culture culture,
            string categoryID,
            string sortBy,
            int startIndex,
            int endIndex,
            BoolFilter isEnabled,
            out int howmanyItems,
            string storeID
            )
        {
            howmanyItems = 0;
            Pair list = (Pair) _dataCache_Product.GetValue(
                GetByCategoryIDSignature,
                _cacheDependency,
                culture.CultureID,
                categoryID,
                sortBy,
                startIndex,
                endIndex,
                isEnabled,
                storeID
                );
            if (list == null)
            {
                list = new Pair();
                list.First = _repository.GetByCategoryID(
                    culture,
                    categoryID,
                    sortBy,
                    startIndex,
                    endIndex,
                    isEnabled,
                    out howmanyItems,
                    storeID
                    );

                list.Second = howmanyItems;
                _dataCache_Product.SetValue( GetByCategoryIDSignature, list, culture.CultureID, categoryID, sortBy, startIndex, endIndex, isEnabled, storeID );
            }
            howmanyItems = ConvertUtilities.ToInt32( list.Second );

            return (IList<Product>) list.First;
        }

        public IList<Product> GetByManufacturerID(
           Culture culture,
           string manufacturerID,
           string sortBy,
           int startIndex,
           int endIndex,
           BoolFilter isEnabled,
           out int howmanyItems,
           string storeID
           )
        {
            howmanyItems = 0;
            Pair list = (Pair) _dataCache_Product.GetValue(
                GetByManufacturerIDSignature,
                _cacheDependency,
                culture.CultureID,
                manufacturerID,
                sortBy,
                startIndex,
                endIndex,
                isEnabled,
                storeID
                );
            if (list == null)
            {
                list = new Pair();
                list.First = _repository.GetByManufacturerID(
                    culture,
                    manufacturerID,
                    sortBy,
                    startIndex,
                    endIndex,
                    isEnabled,
                    out howmanyItems,
                    storeID
                    );

                list.Second = howmanyItems;
                _dataCache_Product.SetValue( GetByManufacturerIDSignature, list, culture.CultureID, manufacturerID, sortBy, startIndex, endIndex, isEnabled, storeID );
            }
            howmanyItems = ConvertUtilities.ToInt32( list.Second );

            return (IList<Product>) list.First;
        }

        private string ConvertSpecificationItemValueListToString( IList<SpecificationItemValue> list )
        {
            string output = string.Empty;
            foreach (SpecificationItemValue item in list)
            {
                output += item.SpecificationItemID + ",";
            }
            return output;
        }

        private string GenerateUniqueStringFromList( IList<string> list )
        {
            List<string> newList = new List<string>( list );
            return string.Join( ",", newList.ToArray() );
        }

        public IList<Product> GetNewArrivalByCategoryDepartmentID(
            Culture culture,
            string[] categoryIDs,
            string[] departmentIDs,
            int limitCount,
            string storeID )
        {
            IList<Product> list =
                (IList<Product>) _dataCache_Product.GetValue(
                GetNewArrivalByCategoryDepartmentIDSignature,
                _cacheDependency,
                culture.CultureID,
                GenerateUniqueStringFromList( categoryIDs ),
                GenerateUniqueStringFromList( departmentIDs ),
                limitCount,
                storeID );
            if (list == null)
            {
                list = _repository.GetNewArrivalByCategoryDepartmentID(
                    culture,
                    categoryIDs,
                    departmentIDs,
                    limitCount,
                    storeID );
                _dataCache_Product.SetValue(
                    GetNewArrivalByCategoryDepartmentIDSignature, list, culture.CultureID,
                    GenerateUniqueStringFromList( categoryIDs ),
                    GenerateUniqueStringFromList( departmentIDs ),
                    limitCount,
                    storeID );
            }
            return list;
        }

        public IList<Product> GetNewArrivalByManufacturerID(
            Culture culture,
            string manufacturerID,
            int limitCount,
            string storeID )
        {
            IList<Product> list =
                (IList<Product>) _dataCache_Product.GetValue(
                GetNewArrivalByManufacturerIDSignature,
                _cacheDependency,
                culture.CultureID,
                manufacturerID,
                limitCount,
                storeID );
            if (list == null)
            {
                list = _repository.GetNewArrivalByManufacturerID(
                    culture,
                    manufacturerID,
                    limitCount,
                    storeID );
                _dataCache_Product.SetValue(
                    GetNewArrivalByManufacturerIDSignature, list, culture.CultureID,
                    manufacturerID,
                    limitCount,
                    storeID );
            }
            return list;
        }

        public IList<Product> GetFacetResultByCategoryID(
            Culture culture,
            string[] categoryIDs,
            string[] departmentIDs,
            string manufacturerID,
            string minPrice,
            string maxPrice,
            IList<SpecificationItemValue> specValueList,
            string sortBy,
            int startIndex,
            int endIndex,
            BoolFilter isEnabled,
            out int howmanyItems,
            string storeID
            )
        {
            howmanyItems = 0;
            Pair list = (Pair) _dataCache_Product.GetValue(
                GetFacetResultByCategoryIDSignature,
                _cacheDependency,
                culture.CultureID,
                GenerateUniqueStringFromList( categoryIDs ),
                GenerateUniqueStringFromList( departmentIDs ),
                manufacturerID,
                minPrice,
                maxPrice,
                ConvertSpecificationItemValueListToString( specValueList ),
                sortBy,
                startIndex,
                endIndex,
                isEnabled,
                storeID
                );
            if (list == null)
            {
                list = new Pair();
                list.First = _repository.GetFacetResultByCategoryID(
                    culture,
                    categoryIDs,
                    departmentIDs,
                    manufacturerID,
                    minPrice,
                    maxPrice,
                    specValueList,
                    sortBy,
                    startIndex,
                    endIndex,
                    isEnabled,
                    out howmanyItems,
                    storeID
                    );

                list.Second = howmanyItems;
                _dataCache_Product.SetValue(
                    GetFacetResultByCategoryIDSignature,
                    list,
                    culture.CultureID,
                    GenerateUniqueStringFromList( categoryIDs ),
                    GenerateUniqueStringFromList( departmentIDs ),
                    minPrice,
                    maxPrice,
                    ConvertSpecificationItemValueListToString( specValueList ),
                    manufacturerID,
                    sortBy,
                    startIndex,
                    endIndex,
                    isEnabled,
                    storeID );
            }
            howmanyItems = ConvertUtilities.ToInt32( list.Second );

            return (IList<Product>) list.First;
        }

        public IList<Product> GetByDepartmentID(
            Culture culture,
            string departmentID,
            string sortBy,
            int startIndex,
            int endIndex,
            BoolFilter isEnabled,
            out int howmanyItems,
            string storeID )
        {
            howmanyItems = 0;
            Pair list = (Pair) _dataCache_Product.GetValue(
                GetByDepartmentIDSignature,
                _cacheDependency,
                culture.CultureID,
                departmentID,
                sortBy,
                startIndex,
                endIndex,
                isEnabled,
                storeID
                );
            if (list == null)
            {
                list = new Pair();
                list.First = _repository.GetByDepartmentID(
                    culture,
                    departmentID,
                    sortBy,
                    startIndex,
                    endIndex,
                    isEnabled,
                    out howmanyItems,
                    storeID
                    );

                list.Second = howmanyItems;
                _dataCache_Product.SetValue( GetByDepartmentIDSignature, list, culture.CultureID, departmentID, sortBy, startIndex, endIndex, isEnabled, storeID );
            }
            howmanyItems = ConvertUtilities.ToInt32( list.Second );

            return (IList<Product>) list.First;
        }

        public IList<Product> GetByDepartmentID(
            Culture culture,
            string[] departmentIDs,
            string sortBy,
            int startIndex,
            int endIndex,
            BoolFilter isEnabled,
            out int howmanyItems,
           string storeID )
        {
            howmanyItems = 0;
            Pair list = (Pair) _dataCache_Product.GetValue(
                GetByDepartmentIDSignature,
                _cacheDependency,
                culture.CultureID,
                string.Join( ",", departmentIDs ),
                sortBy,
                startIndex,
                endIndex,
                isEnabled,
                storeID
                );
            if (list == null)
            {
                list = new Pair();
                list.First = _repository.GetByDepartmentID(
                    culture,
                    departmentIDs,
                    sortBy,
                    startIndex,
                    endIndex,
                    isEnabled,
                    out howmanyItems,
                    storeID
                    );

                list.Second = howmanyItems;
                _dataCache_Product.SetValue( GetByDepartmentIDSignature, list, culture.CultureID, string.Join( ",", departmentIDs ), sortBy, startIndex, endIndex, isEnabled, storeID );
            }
            howmanyItems = ConvertUtilities.ToInt32( list.Second );

            return (IList<Product>) list.First;
        }

        public bool ImageIsUsedByAnotherProduct( string urlLargeImage )
        {
            object value = _dataCache_Product.GetValue( ImageIsUsedByAnotherProductSignature, _cacheDependency, urlLargeImage );
            if (value == null)
            {
                value = _repository.ImageIsUsedByAnotherProduct( urlLargeImage );
                _dataCache_Product.SetValue( ImageIsUsedByAnotherProductSignature, value, urlLargeImage );
            }
            return (bool) value;
        }

        public bool IsRegularImageExisting( string regularImagePath )
        {
            object value = _dataCache_Product.GetValue( IsRegularImageExistingSignature, _cacheDependency, regularImagePath );
            if (value == null)
            {
                value = _repository.IsRegularImageExisting( regularImagePath );
                _dataCache_Product.SetValue( IsRegularImageExistingSignature, value, regularImagePath );
            }
            return (bool) value;
        }

        public bool IsLargeImageExisting( string largeImagePath )
        {
            object value = _dataCache_Product.GetValue( IsLargeImageExistingSignature, _cacheDependency, largeImagePath );
            if (value == null)
            {
                value = _repository.IsLargeImageExisting( largeImagePath );
                _dataCache_Product.SetValue( IsLargeImageExistingSignature, value, largeImagePath );
            }
            return (bool) value;
        }

        public bool IsThumbnailImageExisting( string thumbnailImagePath )
        {
            object value = _dataCache_Product.GetValue( IsThumbnailImageExistingSignature, _cacheDependency, thumbnailImagePath );
            if (value == null)
            {
                value = _repository.IsThumbnailImageExisting( thumbnailImagePath );
                _dataCache_Product.SetValue( IsThumbnailImageExistingSignature, value, thumbnailImagePath );
            }
            return (bool) value;
        }

        public bool IsImageSecondaryExisting( string imageSecondary )
        {
            object value = _dataCache_Product.GetValue( IsImageSecondaryExistingSignature, _cacheDependency, imageSecondary );
            if (value == null)
            {
                value = _repository.IsImageSecondaryExisting( imageSecondary );
                _dataCache_Product.SetValue( IsImageSecondaryExistingSignature, value, imageSecondary );
            }
            return (bool) value;
        }

        public IList<Product> GetAllByBestSelling(
            Culture culture,
            int numberOfResult,
            int outOfStockValue,
            bool useStockInventoryControl,
            string storeID,
            string rootCategoryID,
            bool isIncludeProductKit
            )
        {
            IList<Product> list =
                (IList<Product>) _dataCache_Product.GetValue(
                GetAllByBestSellingSignature,
                _cacheDependency,
                culture.CultureID,
                numberOfResult,
                outOfStockValue,
                useStockInventoryControl,
                storeID,
                rootCategoryID,
                isIncludeProductKit
                );
            if (list == null)
            {
                list = _repository.GetAllByBestSelling(
                    culture,
                    numberOfResult,
                    outOfStockValue,
                    useStockInventoryControl,
                    storeID,
                    rootCategoryID,
                    isIncludeProductKit
                    );
                _dataCache_Product.SetValue(
                    GetAllByBestSellingSignature, list, culture.CultureID, numberOfResult, outOfStockValue, useStockInventoryControl, storeID, rootCategoryID, isIncludeProductKit );
            }
            return list;
        }

        public IList<Product> GetAllByBestSelling(
            Culture culture,
            int numberOfResult,
            int outOfStockValue,
            bool useStockInventoryControl,
            string storeID,
            string rootCategoryID
        )
        {
            IList<Product> list =
                (IList<Product>) _dataCache_Product.GetValue(
                GetAllByBestSellingSignature,
                _cacheDependency,
                culture.CultureID,
                numberOfResult,
                outOfStockValue,
                useStockInventoryControl,
                storeID,
                rootCategoryID
                );
            if (list == null)
            {
                list = _repository.GetAllByBestSelling(
                    culture,
                    numberOfResult,
                    outOfStockValue,
                    useStockInventoryControl,
                    storeID,
                    rootCategoryID
                    );
                _dataCache_Product.SetValue(
                    GetAllByBestSellingSignature, list, culture.CultureID, numberOfResult, outOfStockValue, useStockInventoryControl, storeID, rootCategoryID );
            }
            return list;
        }

        public IList<Product> GetAllByRandom(
            Culture culture,
            int numberOfRandomProducts,
            int outOfStockValue,
            bool useStockInventoryControl,
            string storeID,
            string rootCategoryID,
            bool isIncludeProductKit
            )
        {
            IList<Product> list =
                (IList<Product>) _dataCache_Product.GetValue(
                GetAllByRandomSignature,
                _cacheDependency,
                culture.CultureID,
                numberOfRandomProducts,
                outOfStockValue,
                useStockInventoryControl,
                storeID,
                rootCategoryID,
                isIncludeProductKit
                );
            if (list == null)
            {
                list = _repository.GetAllByRandom(
                    culture,
                    numberOfRandomProducts,
                    outOfStockValue,
                    useStockInventoryControl,
                    storeID,
                    rootCategoryID,
                    isIncludeProductKit
                    );
                _dataCache_Product.SetValue(
                    GetAllByRandomSignature, list, culture.CultureID, numberOfRandomProducts, outOfStockValue, useStockInventoryControl, storeID, rootCategoryID, isIncludeProductKit );
            }
            return list;
        }

        public IList<Product> GetAllByRandom(
            Culture culture,
            int numberOfRandomProducts,
            int outOfStockValue,
            bool useStockInventoryControl,
            string storeID,
            string rootCategoryID
        )
        {
            IList<Product> list =
                (IList<Product>) _dataCache_Product.GetValue(
                GetAllByRandomSignature,
                _cacheDependency,
                culture.CultureID,
                numberOfRandomProducts,
                outOfStockValue,
                useStockInventoryControl,
                storeID,
                rootCategoryID
                );
            if (list == null)
            {
                list = _repository.GetAllByRandom(
                    culture,
                    numberOfRandomProducts,
                    outOfStockValue,
                    useStockInventoryControl,
                    storeID,
                    rootCategoryID
                    );
                _dataCache_Product.SetValue(
                    GetAllByRandomSignature, list, culture.CultureID, numberOfRandomProducts, outOfStockValue, useStockInventoryControl, storeID, rootCategoryID );
            }
            return list;
        }

        public IList<Product> GetAllByNewArrival(
            Culture culture,
            int numberOfRandomProducts,
            int outOfStockValue,
            bool useStockInventoryControl,
            string storeID,
            string rootCategoryID,
            bool isIncludeProductKit
            )
        {
            IList<Product> list =
                (IList<Product>) _dataCache_Product.GetValue(
                GetAllByNewArrivalSignature,
                _cacheDependency,
                culture.CultureID,
                numberOfRandomProducts,
                outOfStockValue,
                useStockInventoryControl,
                storeID,
                rootCategoryID,
                isIncludeProductKit
                );
            if (list == null)
            {
                list = _repository.GetAllByNewArrival(
                    culture,
                    numberOfRandomProducts,
                    outOfStockValue,
                    useStockInventoryControl,
                    storeID,
                    rootCategoryID,
                    isIncludeProductKit
                    );
                _dataCache_Product.SetValue(
                    GetAllByNewArrivalSignature, list, culture.CultureID, numberOfRandomProducts, outOfStockValue, useStockInventoryControl, storeID, rootCategoryID, isIncludeProductKit );
            }
            return list;
        }

        public IList<Product> GetAllByNewArrival(
            Culture culture,
            int numberOfRandomProducts,
            int outOfStockValue,
            bool useStockInventoryControl,
            string storeID,
            string rootCategoryID
        )
        {
            IList<Product> list =
                (IList<Product>) _dataCache_Product.GetValue(
                GetAllByNewArrivalSignature,
                _cacheDependency,
                culture.CultureID,
                numberOfRandomProducts,
                outOfStockValue,
                useStockInventoryControl,
                storeID,
                rootCategoryID
                );
            if (list == null)
            {
                list = _repository.GetAllByNewArrival(
                    culture,
                    numberOfRandomProducts,
                    outOfStockValue,
                    useStockInventoryControl,
                    storeID,
                    rootCategoryID
                    );
                _dataCache_Product.SetValue(
                    GetAllByNewArrivalSignature, list, culture.CultureID, numberOfRandomProducts, outOfStockValue, useStockInventoryControl, storeID, rootCategoryID );
            }
            return list;
        }

        public IList<Product> GetAllByTodaySpecial(
            Culture culture, int outOfStockValue, bool useStockInventoryControl, string storeID, string rootCategoryID, bool isIncludeProductKit )
        {
            IList<Product> list =
                (IList<Product>) _dataCache_Product.GetValue(
                GetAllByTodaySpecialSignature,
                _cacheDependency,
                culture.CultureID,
                outOfStockValue,
                useStockInventoryControl,
                storeID,
                rootCategoryID,
                 isIncludeProductKit
                );
            if (list == null)
            {
                list = _repository.GetAllByTodaySpecial(
                    culture,
                    outOfStockValue,
                    useStockInventoryControl,
                    storeID,
                    rootCategoryID, isIncludeProductKit );
                _dataCache_Product.SetValue(
                    GetAllByTodaySpecialSignature, list, culture.CultureID, outOfStockValue, useStockInventoryControl, storeID, rootCategoryID, isIncludeProductKit );
            }
            return list;
        }

        public IList<Product> GetAllByTodaySpecial(
            Culture culture, int outOfStockValue, bool useStockInventoryControl, string storeID, string rootCategoryID )
        {
            IList<Product> list =
                (IList<Product>) _dataCache_Product.GetValue(
                GetAllByTodaySpecialSignature,
                _cacheDependency,
                culture.CultureID,
                outOfStockValue,
                useStockInventoryControl,
                storeID,
                rootCategoryID
                );
            if (list == null)
            {
                list = _repository.GetAllByTodaySpecial(
                    culture,
                    outOfStockValue,
                    useStockInventoryControl,
                    storeID,
                    rootCategoryID );
                _dataCache_Product.SetValue(
                    GetAllByTodaySpecialSignature, list, culture.CultureID, outOfStockValue, useStockInventoryControl, storeID, rootCategoryID );
            }
            return list;
        }

        public IList<Product> GetAllByTaxClassID( Culture culture, string taxClassID, string storeID )
        {
            IList<Product> list =
                (IList<Product>) _dataCache_Product.GetValue(
                GetAllByTaxClassIDSignature,
                _cacheDependency,
                culture.CultureID,
                taxClassID,
                storeID
                );
            if (list == null)
            {
                list = _repository.GetAllByTaxClassID( culture, taxClassID, storeID );
                _dataCache_Product.SetValue( GetAllByTaxClassIDSignature, list, culture.CultureID, taxClassID, storeID );
            }

            return list;
        }

        public IList<Product> GetAllBySubscriptionLevelID(
            Culture culture,
            string subscriptionID,
            string storeID )
        {
            IList<Product> list =
                (IList<Product>) _dataCache_Product.GetValue(
                GetAllBySubscriptionLevelIDSignature,
                _cacheDependency,
                culture.CultureID,
                subscriptionID,
                storeID
                );

            if (list == null)
            {
                list = _repository.GetAllBySubscriptionLevelID( culture, subscriptionID, storeID );
                _dataCache_Product.SetValue( GetAllBySubscriptionLevelIDSignature, list, culture.CultureID, subscriptionID, storeID );
            }

            return list;
        }

        public int GetAllProductCount()
        {
            object productCount = _dataCache_Product.GetValue( GetAllProductCountSignature, _cacheDependency );
            if (productCount == null)
            {
                productCount = _repository.GetAllProductCount();
                _dataCache_Product.SetValue( GetAllProductCountSignature, Shared.Utilities.ConvertUtilities.ToInt32( productCount ) );
            }
            return Shared.Utilities.ConvertUtilities.ToInt32( productCount );
        }

        public decimal GetMaxPriceByCategoryDepartment(
            Culture culture,
            string[] categoryIDs,
            string[] departmentIDs,
            BoolFilter isEnabled,
            string storeID
            )
        {
            object result = _dataCache_Product.GetValue( GetMaxPriceByCategoryDepartmentSignature, _cacheDependency,
                culture.CultureID,
                string.Join( ",", categoryIDs ),
                string.Join( ",", departmentIDs ),
                isEnabled,
                storeID
                );
            if (result == null)
            {
                result = _repository.GetMaxPriceByCategoryDepartment( culture, categoryIDs, departmentIDs, isEnabled, storeID );
                _dataCache_Product.SetValue( GetMaxPriceByCategoryDepartmentSignature,
                    Shared.Utilities.ConvertUtilities.ToDecimal( result ),
                    culture.CultureID,
                    string.Join( ",", categoryIDs ),
                    string.Join( ",", departmentIDs ),
                    isEnabled,
                    storeID );
            }

            return Shared.Utilities.ConvertUtilities.ToDecimal( result );
        }

        public int GetFacetCount(
            Culture culture,
            string[] categoryIDs,
            string[] departmentIDs,
            string manufacturerID,
            string minPrice,
            string maxPrice,
            IList<SpecificationItemValue> specValueList,
            string sortBy,
            BoolFilter isEnabled,
            string storeID
            )
        {
            object productCount = _dataCache_Product.GetValue( GetFacetCountSignature,
                _cacheDependency,
                culture.CultureID,
                string.Join( ",", categoryIDs ),
                string.Join( ",", departmentIDs ),
                minPrice,
                maxPrice,
                ConvertSpecificationItemValueListToString( specValueList ),
                sortBy,
                isEnabled,
                storeID
            );
            if (productCount == null)
            {
                productCount = _repository.GetFacetCount( culture, categoryIDs, departmentIDs, manufacturerID, minPrice, maxPrice, specValueList, sortBy, isEnabled, storeID );
                _dataCache_Product.SetValue( GetFacetCountSignature, productCount,
                    culture.CultureID,
                     string.Join( ",", categoryIDs ),
                    string.Join( ",", departmentIDs ),
                    manufacturerID,
                    minPrice,
                    maxPrice,
                    ConvertSpecificationItemValueListToString( specValueList ),
                    sortBy,
                    isEnabled,
                    storeID );
            }

            return Shared.Utilities.ConvertUtilities.ToInt32( productCount );
        }

        public int GetAllProductCountBySku( string sku )
        {
            object productCount = _dataCache_Product.GetValue( GetAllProductCountBySkuSignature, _cacheDependency, sku );
            if (productCount == null)
            {
                productCount = _repository.GetAllProductCountBySku( sku );
                _dataCache_Product.SetValue( GetAllProductCountBySkuSignature, Shared.Utilities.ConvertUtilities.ToInt32( productCount ), sku );
            }
            return Shared.Utilities.ConvertUtilities.ToInt32( productCount );
        }

        public int GetAllProductInactiveCount()
        {
            object productCount = _dataCache_Product.GetValue( GetAllProductInactiveCountSignature, _cacheDependency );
            if (productCount == null)
            {
                productCount = _repository.GetAllProductInactiveCount();
                _dataCache_Product.SetValue( GetAllProductInactiveCountSignature, Shared.Utilities.ConvertUtilities.ToInt32( productCount ) );
            }
            return Shared.Utilities.ConvertUtilities.ToInt32( productCount );
        }

        public int GetAllProductInLowStockCount( int lowStock, bool useStockInventoryControl )
        {
            object productCount = _dataCache_Product.GetValue( GetAllProductInLowStockCountSignature, _cacheDependency, lowStock, useStockInventoryControl );
            if (productCount == null)
            {
                productCount = _repository.GetAllProductInLowStockCount( lowStock, useStockInventoryControl );
                _dataCache_Product.SetValue( GetAllProductInLowStockCountSignature, Shared.Utilities.ConvertUtilities.ToInt32( productCount ), lowStock, useStockInventoryControl );
            }
            return Shared.Utilities.ConvertUtilities.ToInt32( productCount );
        }

        public int GetProductCountByCategoryID( string categoryID, BoolFilter isEnabled )
        {
            object productCount = _dataCache_Product.GetValue( GetProductCountByCategoryIDSignature, _cacheDependency, categoryID, isEnabled );
            if (productCount == null)
            {
                productCount = _repository.GetProductCountByCategoryID( categoryID, isEnabled );
                _dataCache_Product.SetValue( GetProductCountByCategoryIDSignature, Shared.Utilities.ConvertUtilities.ToInt32( productCount ), categoryID, isEnabled );
            }
            return Shared.Utilities.ConvertUtilities.ToInt32( productCount );
        }

        public int GetProductCountByCategoryID( string[] categoryIDs )
        {
            object productCount = _dataCache_Product.GetValue( GetProductCountByCategoryIDSignature, _cacheDependency, string.Join( ",", categoryIDs ) );
            if (productCount == null)
            {
                productCount = _repository.GetProductCountByCategoryID( categoryIDs );
                _dataCache_Product.SetValue( GetProductCountByCategoryIDSignature, Shared.Utilities.ConvertUtilities.ToInt32( productCount ), string.Join( ",", categoryIDs ) );
            }
            return Shared.Utilities.ConvertUtilities.ToInt32( productCount );
        }

        public int GetProductCountByDepartmentID( string departmentID, BoolFilter isEnabled )
        {
            object productCount = _dataCache_Product.GetValue( GetProductCountByDepartmentIDSignature, _cacheDependency, departmentID, isEnabled );
            if (productCount == null)
            {
                productCount = _repository.GetProductCountByDepartmentID( departmentID, isEnabled );
                _dataCache_Product.SetValue( GetProductCountByDepartmentIDSignature, Shared.Utilities.ConvertUtilities.ToInt32( productCount ), departmentID, isEnabled );
            }
            return Shared.Utilities.ConvertUtilities.ToInt32( productCount );
        }

        public int GetProductCountByDepartmentID( string[] departmentIDs )
        {
            object productCount = _dataCache_Product.GetValue( GetProductCountByDepartmentIDSignature, _cacheDependency, string.Join( ",", departmentIDs ) );
            if (productCount == null)
            {
                productCount = _repository.GetProductCountByDepartmentID( departmentIDs );
                _dataCache_Product.SetValue( GetProductCountByDepartmentIDSignature, Shared.Utilities.ConvertUtilities.ToInt32( productCount ), string.Join( ",", departmentIDs ) );
            }
            return Shared.Utilities.ConvertUtilities.ToInt32( productCount );
        }

        public int GetProductCountByManufacturerID( string manufacturerID, BoolFilter isEnabled )
        {
            object productCount = _dataCache_Product.GetValue( GetProductCountByManufacturerIDSignature, _cacheDependency, manufacturerID, isEnabled );
            if (productCount == null)
            {
                productCount = _repository.GetProductCountByManufacturerID( manufacturerID, isEnabled );
                _dataCache_Product.SetValue( GetProductCountByManufacturerIDSignature, Shared.Utilities.ConvertUtilities.ToInt32( productCount ), manufacturerID, isEnabled );
            }
            return Shared.Utilities.ConvertUtilities.ToInt32( productCount );
        }

        public Dictionary<string, int> GetProductCountGroupByCategoryID()
        {
            object categoryID = _dataCache_Product.GetValue( GetProductCountGroupByCategoryIDSignature, _cacheDependency );
            if (categoryID == null)
            {
                categoryID = _repository.GetProductCountGroupByCategoryID();
                _dataCache_Product.SetValue( GetProductCountGroupByCategoryIDSignature, categoryID );
            }
            return (Dictionary<string, int>) categoryID;
        }

        public Dictionary<string, int> GetProductCountGroupByDepartmentID()
        {
            object departmentID = _dataCache_Product.GetValue( GetProductCountGroupByDepartmentIDSignature, _cacheDependency );
            if (departmentID == null)
            {
                departmentID = _repository.GetProductCountGroupByDepartmentID();
                _dataCache_Product.SetValue( GetProductCountGroupByDepartmentIDSignature, departmentID );
            }
            return (Dictionary<string, int>) departmentID;
        }

        public IList<TableSchemaItem> GetTableSchema()
        {
            IList<TableSchemaItem> list = (IList<TableSchemaItem>) _dataCache_Product.GetValue(
                GetTableSchemaSignature,
                _cacheDependency );

            if (list == null)
            {
                list = _repository.GetTableSchema();
                _dataCache_Product.SetValue( GetTableSchemaSignature, list );
            }
            return list;
        }

        public bool HasProductInDiscountGroup( string discountgroupID )
        {
            object result = _dataCache_Product.GetValue( HasProductInDiscountGroupSignature, _cacheDependency, discountgroupID );
            if (result == null)
            {
                result = _repository.HasProductInDiscountGroup( discountgroupID );
                _dataCache_Product.SetValue( HasProductInDiscountGroupSignature, result, discountgroupID );
            }
            return ConvertUtilities.ToBoolean( result );
        }

        public IList<Product> SearchProduct(
            Culture culture,
            string categoryID,
            string sortBy,
            SearchFilter searchFilter,
            int startIndex,
            int endIndex,
            out int howManyItems,
            string storeID,
            string rootCategoryID
            )
        {
            howManyItems = 0;
            Pair list = (Pair) _dataCache_Product.GetValue(
                SearchProductSignature,
                _cacheDependency,
                culture.CultureID,
                categoryID,
                sortBy,
                searchFilter.ToString(),
                startIndex,
                endIndex,
                storeID,
                rootCategoryID
                );

            if (list == null)
            {
                list = new Pair();
                list.First = _repository.SearchProduct(
                    culture, categoryID, sortBy, searchFilter, startIndex, endIndex, out howManyItems, storeID, rootCategoryID );
                list.Second = howManyItems;
                _dataCache_Product.SetValue(
                    SearchProductSignature, list, culture.CultureID, categoryID, sortBy, searchFilter.ToString(), startIndex, endIndex, storeID, rootCategoryID );
            }
            howManyItems = ConvertUtilities.ToInt32( list.Second );

            return (IList<Product>) list.First;
        }

        public IList<Product> QuickSearch(
            Culture culture,
            string searchTerm,
            string[] searchFieldList,
            string sortBy,
            int startIndex,
            int endIndex,
            out int howManyItems,
            string storeID,
            string rootCategoryID,
            string searchType
            )
        {
            howManyItems = 0;
            Pair list = (Pair) _dataCache_Product.GetValue(
                QuickSearchSignature,
                _cacheDependency,
                culture.CultureID,
                searchTerm,
                string.Join( ",", searchFieldList ),
                sortBy,
                startIndex,
                endIndex,
                storeID,
                rootCategoryID,
                searchType
                );
            if (list == null)
            {
                list = new Pair();
                list.First = _repository.QuickSearch(
                    culture, searchTerm, searchFieldList, sortBy, startIndex, endIndex, out howManyItems, storeID, rootCategoryID, searchType );
                list.Second = howManyItems;
                _dataCache_Product.SetValue( QuickSearchSignature, list, culture.CultureID, searchTerm, string.Join( ",", searchFieldList ), sortBy, startIndex, endIndex, storeID, rootCategoryID, searchType );
            }
            howManyItems = ConvertUtilities.ToInt32( list.Second );

            return (IList<Product>) list.First;
        }

        public IList<Product> AdvancedSearch(
            Culture culture,
            string categoryIDs,
            string sortBy,
            string searchTerm,
            string price1,
            string price2,
            string[] searchFieldList,
            int startIndex,
            int endIndex,
            out int howManyItems,
            string storeID,
            string rootCategoryID,
            string searchType
            )
        {
            howManyItems = 0;
            Pair list = (Pair) _dataCache_Product.GetValue(
               AdvancedSearchSignature,
               _cacheDependency,
               culture.CultureID,
               categoryIDs,
               sortBy,
               searchTerm,
               price1,
               price2,
               string.Join( ",", searchFieldList ),
               startIndex,
               endIndex,
               storeID,
               rootCategoryID,
               searchType
               );
            if (list == null)
            {
                list = new Pair();
                list.First = _repository.AdvancedSearch(
                    culture, categoryIDs, sortBy, searchTerm, price1, price2, searchFieldList, startIndex, endIndex, out howManyItems, storeID, rootCategoryID, searchType );
                list.Second = howManyItems;
                _dataCache_Product.SetValue(
                    AdvancedSearchSignature, list, culture.CultureID, categoryIDs, sortBy, searchTerm, price1, price2, string.Join( ",", searchFieldList ), startIndex, endIndex, storeID, rootCategoryID, searchType );
            }
            howManyItems = ConvertUtilities.ToInt32( list.Second );

            return (IList<Product>) list.First;
        }

        public IList<Product> AdvancedSearch(
            Culture culture,
            string categoryIDs,
            string departmentIDs,
            string manufacturerID,
            string sortBy,
            string searchTerm,
            string price1,
            string price2,
            string[] searchFieldList,
            int startIndex,
            int endIndex,
            out int howManyItems,
            string storeID,
            string rootCategoryID,
            string rootDepartmentID,
            string searchType
            )
        {
            howManyItems = 0;
            Pair list = (Pair) _dataCache_Product.GetValue(
               AdvancedSearchSignature,
               _cacheDependency,
               culture.CultureID,
               categoryIDs,
               departmentIDs,
               manufacturerID,
               sortBy,
               searchTerm,
               price1,
               price2,
               string.Join( ",", searchFieldList ),
               startIndex,
               endIndex,
               storeID,
               rootCategoryID,
               rootDepartmentID,
               searchType
               );
            if (list == null)
            {
                list = new Pair();
                list.First = _repository.AdvancedSearch(
                    culture, categoryIDs, departmentIDs, manufacturerID, sortBy, searchTerm, price1, price2, searchFieldList, startIndex, endIndex, out howManyItems, storeID, rootCategoryID, rootDepartmentID, searchType );
                list.Second = howManyItems;
                _dataCache_Product.SetValue(
                    AdvancedSearchSignature, list, culture.CultureID, categoryIDs, departmentIDs, manufacturerID, sortBy, searchTerm, price1, price2, string.Join( ",", searchFieldList ), startIndex, endIndex, storeID, rootCategoryID, rootDepartmentID, searchType );
            }
            howManyItems = ConvertUtilities.ToInt32( list.Second );

            return (IList<Product>) list.First;
        }

        public IList<Product> AdvancedSearch(
            Culture culture,
            string categoryIDs,
            string sortBy,
            string searchTerm,
            string price1,
            string price2,
            string[] searchFieldList,
            int startIndex,
            int endIndex,
            out int howManyItems,
            string storeID,
            string rootCategoryID,
            bool isIncludeProductKit,
            string searchType
        )
        {
            howManyItems = 0;
            Pair list = (Pair) _dataCache_Product.GetValue(
               AdvancedSearchSignature,
               _cacheDependency,
               culture.CultureID,
               categoryIDs,
               sortBy,
               searchTerm,
               price1,
               price2,
               string.Join( ",", searchFieldList ),
               startIndex,
               endIndex,
               storeID,
               rootCategoryID,
               isIncludeProductKit,
               searchType
               );
            if (list == null)
            {
                list = new Pair();
                list.First = _repository.AdvancedSearch(
                    culture, categoryIDs, sortBy, searchTerm, price1, price2, searchFieldList, startIndex, endIndex, out howManyItems, storeID, rootCategoryID, isIncludeProductKit, searchType );
                list.Second = howManyItems;
                _dataCache_Product.SetValue(
                    AdvancedSearchSignature, list, culture.CultureID, categoryIDs, sortBy, searchTerm, price1, price2, string.Join( ",", searchFieldList ), startIndex, endIndex, storeID, rootCategoryID, isIncludeProductKit, searchType );
            }
            howManyItems = ConvertUtilities.ToInt32( list.Second );

            return (IList<Product>) list.First;
        }

        public IList<Product> AdvancedSearchDepartment(
            Culture culture,
            string departmentIDs,
            string sortBy,
            string searchTerm,
            string price1,
            string price2,
            string[] searchFieldList,
            int startIndex,
            int endIndex,
            out int howManyItems,
            string storeID,
            string rootDepartmentID,
            string searchType
            )
        {
            howManyItems = 0;
            Pair list = (Pair) _dataCache_Product.GetValue(
               AdvancedSearchSignature,
               _cacheDependency,
               culture.CultureID,
               departmentIDs,
               sortBy,
               searchTerm,
               price1,
               price2,
               string.Join( ",", searchFieldList ),
               startIndex,
               endIndex,
               storeID,
               rootDepartmentID,
               searchType
               );
            if (list == null)
            {
                list = new Pair();
                list.First = _repository.AdvancedSearchDepartment(
                    culture, departmentIDs, sortBy, searchTerm, price1, price2, searchFieldList, startIndex, endIndex, out howManyItems, storeID, rootDepartmentID, searchType );
                list.Second = howManyItems;
                _dataCache_Product.SetValue(
                    AdvancedSearchSignature, list, culture.CultureID, departmentIDs, sortBy, searchTerm, price1, price2, string.Join( ",", searchFieldList ), startIndex, endIndex, storeID, rootDepartmentID, searchType );
            }
            howManyItems = ConvertUtilities.ToInt32( list.Second );

            return (IList<Product>) list.First;
        }

        public DataTable GetStockOptionLine( Culture culture, string[] optionGroupIDs )
        {
            DataTable table = (DataTable) _dataCache_Product.GetValue(
                GetStockOptionLineSignature,
                _cacheDependency,
                culture.CultureID,
                optionGroupIDs );

            if (table == null)
            {
                table = _repository.GetStockOptionLine( culture, optionGroupIDs );
                _dataCache_Product.SetValue( GetStockOptionLineSignature, table, culture.CultureID, optionGroupIDs );
            }
            return table;
        }

        public IList<String> GetProductIDListByCategoryDepartment(
            string[] categoryIDs,
            string[] departmentIDs,
            string manufacturerID,
            BoolFilter isEnabled,
            string storeID )
        {
            IList<String> list = (IList<String>) _dataCache_Product.GetValue(
                GetProductIDListByCategoryDepartmentSignature,
                _cacheDependency,
                string.Join( ",", categoryIDs ),
                string.Join( ",", departmentIDs ),
                manufacturerID,
                isEnabled,
                storeID );

            if (list == null)
            {
                list = _repository.GetProductIDListByCategoryDepartment(
                    categoryIDs,
                    departmentIDs,
                    manufacturerID,
                    isEnabled,
                    storeID );

                _dataCache_Product.SetValue( GetProductIDListByCategoryDepartmentSignature,
                    list,
                    string.Join( ",", categoryIDs ),
                    string.Join( ",", departmentIDs ),
                    manufacturerID,
                    isEnabled,
                    storeID );
            }

            return list;
        }


        public IList<ProductPrice> GetProductPrices( string productID )
        {
            IList<ProductPrice> list =
               (IList<ProductPrice>) _dataCache_Product.GetValue(
                   GetProductPricesSignature,
                   _cacheDependency,
                   productID );

            if (list == null)
            {
                list = _repository.GetProductPrices( productID );
                _dataCache_Product.SetValue( GetProductPricesSignature, list, productID );
            }

            return list;
        }

        public IList<ProductPrice> GetDefaultProductPriceList()
        {
            IList<ProductPrice> list =
                (IList<ProductPrice>) _dataCache_Product.GetValue(
                GetDefaultProductPriceListSignature,
                _cacheDependency );

            if (list == null)
            {
                list = _repository.GetDefaultProductPriceList();
                _dataCache_Product.SetValue( GetDefaultProductPriceListSignature, list );
            }

            return list;
        }

        public ProductPrice GetDefaultProductPrice( string productID )
        {
            ProductPrice productPrice = (ProductPrice) _dataCache_Product.GetValue(
                GetDefaultProductPriceSignature,
                _cacheDependency,
                productID );

            if (productPrice == new ProductPrice())
            {
                productPrice = _repository.GetDefaultProductPrice( productID );
                _dataCache_Product.SetValue( GetProductPricesSignature, productPrice, productID );
            }
            return productPrice;
        }

        public void DeleteProductPricesByStoreID( string storeID )
        {
            _repository.DeleteProductPricesByStoreID( storeID );
            _dataCache_Product.ClearRepositoryCache();
            _dataCache_OptionGroup.ClearRepositoryCache();
            _dataCache_GoogleFeedMapping.ClearRepositoryCache();
            _dataCache_SpecificationItem.ClearRepositoryCache();
        }

        public void CreateProductPricesFromDefaultByStoreID( string storeID )
        {
            _repository.CreateProductPricesFromDefaultByStoreID( storeID );
            _dataCache_Product.ClearRepositoryCache();
            _dataCache_OptionGroup.ClearRepositoryCache();
            _dataCache_GoogleFeedMapping.ClearRepositoryCache();
            _dataCache_SpecificationItem.ClearRepositoryCache();
        }

        public IList<Product> GetByIsQBExported(
            Culture culture,
            bool isQBExported,
            string rootCategoryID,
            string storeID,
            string existList )
        {
            IList<Product> list = _repository.GetByIsQBExported( culture
, isQBExported, rootCategoryID, storeID, existList );
            return list;
        }

        public IList<Product> GetAllQBProduct( Culture culture, string rootCategoryID, string storeID )
        {
            IList<Product> list = _repository.GetAllQBProduct( culture, rootCategoryID, storeID );
            return list;
        }

        public void UpdateQBData( string productID, string qbListID, bool isQBExported )
        {
            _repository.UpdateQBData( productID, qbListID, isQBExported );
            _dataCache_Product.ClearRepositoryCache();
            _dataCache_OptionGroup.ClearRepositoryCache();
            _dataCache_GoogleFeedMapping.ClearRepositoryCache();
            _dataCache_SpecificationItem.ClearRepositoryCache();
        }

        public void SetDefaultQBData( string qbListID )
        {
            _repository.SetDefaultQBData( qbListID );
            _dataCache_Product.ClearRepositoryCache();
            _dataCache_OptionGroup.ClearRepositoryCache();
            _dataCache_GoogleFeedMapping.ClearRepositoryCache();
            _dataCache_SpecificationItem.ClearRepositoryCache();
        }

        public int CountQuickSearchResult(
            Culture culture,
            string searchTerm,
            string[] searchFieldList,
            string sortBy,
            string storeID,
            string rootCategoryID,
            string searchType
            )
        {
            return _repository.CountQuickSearchResult( culture, searchTerm, searchFieldList, sortBy, storeID, rootCategoryID, searchType );
        }


        public Product GetOneLargeProduct( Culture culture, string productID, string storeID )
        {
            Product product = _repository.GetOneLargeProduct( culture, productID, storeID );
            return product;
        }

        public Product CreateLargeProductItem(
            Product product, ProductLocale locale, ArrayList categoryVisibleIDList, bool processImage, bool isUseOption )
        {
            Product newProduct = _repository.CreateLargeProductItem( product, locale, categoryVisibleIDList, processImage, isUseOption );
            return newProduct;
        }

        public Product UpdateLargeProductItem( Product product, ArrayList categoryVisibleIDList, bool processImage, bool isUseOption )
        {
            Product newProduct = _repository.UpdateLargeProductItem( product, categoryVisibleIDList, processImage, isUseOption );
            return newProduct;
        }


        public Product GetOneBySku( Culture culture, string storeID, string sku )
        {
            Product product = _repository.GetOneBySku( culture, storeID, sku );

            return product;
        }

        public Product GetOneByUrlName( Culture culture, string storeID, string name )
        {
            Product product = _repository.GetOneByUrlName( culture, storeID, name );

            return product;
        }

        public IList<Product> GetProductByName( Culture culture, string storeID, string name )
        {
            return _repository.GetProductByName( culture, storeID, name );
        }

        public bool IsUseCustomPrice( string productID )
        {
            return _repository.IsUseCustomPrice( productID );
        }

        public IList<Product> GetAllProductKit( Culture culture )
        {
            IList<Product> list =
                (IList<Product>) _dataCache_Product.GetValue(
                    GetAllProductKitSignature,
                    _cacheDependency,
                    culture.CultureID
                    );

            if (list == null)
            {
                list = _repository.GetAllProductKit( culture );

                _dataCache_Product.SetValue( GetAllProductKitSignature, list, culture.CultureID );
            }

            return list;
        }

        public IList<Product> GetProductKitItems( Culture culture, string ProductKitGroupID, string sortBy )
        {
            IList<Product> list =
                (IList<Product>) _dataCache_Product.GetValue(
                    GetProductKitItemsSignature,
                    _cacheDependency,
                    culture.CultureID,
                    ProductKitGroupID,
                    sortBy
                    );

            if (list == null)
            {
                list = _repository.GetProductKitItems( culture, ProductKitGroupID, sortBy );

                _dataCache_Product.SetValue( GetAllProductKitSignature, list, culture.CultureID, ProductKitGroupID, sortBy );
            }

            return list;
        }

        public IList<ProductSpecification> GetAllProductSpecifications()
        {
            IList<ProductSpecification> list =
                (IList<ProductSpecification>) _dataCache_Product.GetValue(
                    GetAllProductSpecificationsSignature,
                    _cacheDependency
                    );

            if (list == null)
            {
                list = _repository.GetAllProductSpecifications();

                _dataCache_Product.SetValue( GetAllProductSpecificationsSignature, list );
            }

            return list;
        }

        public bool CheckIsProductKitByProductID( string productID )
        {
            object value = _dataCache_Product.GetValue( CheckIsProductKitByProductIDSignature, _cacheDependency, productID );

            if (value == null)
            {
                value = _repository.CheckIsProductKitByProductID( productID );

                _dataCache_Product.SetValue( CheckIsProductKitByProductIDSignature, value, productID );
            }

            return (bool) value;
        }

        public void IncreaseViewCount( string productID )
        {
            _repository.IncreaseViewCount( productID );
        }

        public Dictionary<string, string> GetMultipleCategoryProduct()
        {
            object value = _dataCache_Product.GetValue( GetMultipleCategoryProductSignature, _cacheDependency );
            if (value == null)
            {
                value = _repository.GetMultipleCategoryProduct();
                _dataCache_Product.SetValue( GetMultipleCategoryProductSignature, value );
            }
            return (Dictionary<string, string>) value;
        }


        public Dictionary<string, string> GetMultipleDepartmentProduct()
        {
            object value = _dataCache_Product.GetValue( GetMultipleDepartmentProductSignature, _cacheDependency );
            if (value == null)
            {
                value = _repository.GetMultipleDepartmentProduct();
                _dataCache_Product.SetValue( GetMultipleDepartmentProductSignature, value );
            }
            return (Dictionary<string, string>) value;
        }

        public IList<Product> GetProductNewArrival( Culture culture, string storeID, int limitResult, string sortBy, BoolFilter isEnabled,
            int startIndex, int endIndex, out int howManyItems )
        {
            howManyItems = 0;
            Pair list = (Pair) _dataCache_Product.GetValue(
                GetProductNewArrivalSignature,
                _cacheDependency,
                culture.CultureID,
                storeID,
                limitResult,
                sortBy,
                isEnabled.ToString(),
                startIndex,
                endIndex
            );

            if (list == null)
            {
                list = new Pair();

                list.First = _repository.GetProductNewArrival(
                    culture,
                    storeID,
                    limitResult,
                    sortBy,
                    isEnabled,
                    startIndex,
                    endIndex,
                    out howManyItems
                );

                list.Second = howManyItems;

                _dataCache_Product.SetValue(
                    GetProductNewArrivalSignature,
                    list,
                    culture.CultureID,
                    storeID,
                    limitResult,
                    sortBy,
                    isEnabled.ToString(),
                    startIndex,
                    endIndex );
            }

            howManyItems = ConvertUtilities.ToInt32( list.Second );
            return (IList<Product>) list.First;
        }

        #endregion
    }
}
