using System;
using System.Data;
using System.Configuration;
using Vevo.Domain;
using Vevo.Domain.Configurations;
using Vevo.Domain.Orders;
using Vevo.Domain.Users;

namespace Vevo.Domain.Shipping.RealTime
{
    /// <summary>
    /// Summary description for RealTimeShippingMethod
    /// </summary>
    public class RealTimeShippingMethod : ShippingMethod
    {
        #region Private

        private ShippingChoice _selectedShippingChoice;


        private bool IsEnabled( string shippingID )
        {
            ShippingOption shippingOption =
                DataAccessContext.ShippingOptionRepository.GetOne( Culture.Null, shippingID );
            return shippingOption.IsEnabled;
        }

        private decimal GetHandlingFee( string shippingID )
        {
            ShippingOption shippingOption =
                DataAccessContext.ShippingOptionRepository.GetOne( Culture.Null, shippingID );
            return shippingOption.HandlingFee;
        }

        #endregion


        #region Constructors

        public RealTimeShippingMethod(
            ShippingOption shippingOption,
            ShippingChoice selectedShippingChoice )

            : base( shippingOption )
        {
            _selectedShippingChoice = selectedShippingChoice;
        }

        #endregion


        #region Public Properties

        public override string Name
        {
            get
            {
                return _selectedShippingChoice.Name;
            }
        }

        #endregion


        #region Public Methods

        public override decimal GetShippingCost(
            CartItemGroup cartItemGroup,
            WholesaleStatus wholesaleStatus,
            decimal discount )
        {
            return _selectedShippingChoice.ShippingCost;
        }

        public override string GetOnlineTrackingName()
        {
            return ShippingOption.ShippingOptionType.TypeName;
        }

        #endregion
    }
}
