using System;
using System.Collections.Generic;
using System.Text;
using Vevo.Domain.Configurations;
using Vevo.Domain.Contents;
using Vevo.Domain.EmailTemplates;
using Vevo.Domain.Base;

namespace Vevo.Domain.Stores
{
    public class Store : ICloneable
    {
        #region Private

        private static String[] _configsSingleCultureList = new String[] {
            // Store Display config
            "StoreTheme",
            "MobileTheme",
            "RootCategory",
            "RootDepartment",
            "DefaultCategoryListLayout",
            "DefaultDepartmentListLayout",
            "DefaultProductListLayout",
            "DefaultProductDetailsLayout",
            "ProductItemsPerPage",
            "CategoryItemsPerPage",
            "ManufacturerItemsPerPage",
            "NumberOfSubCategoryMenuColumn",
            "NumberOfSubCategoryMenuItem",
            "DepartmentItemsPerPage",
            "NumberOfProductColumn",
            "NumberOfCategoryColumn",
            "NumberOfDepartmentColumn",
            "CategoryMenuType",
            "DepartmentMenuType",
            "CategoryMenuLevel",
            "DepartmentMenuLevel",
            "CategoryDynamicDropDownDisplay",
            "DepartmentDynamicDropDownDisplay",
            "ManufacturerDynamicDropDownDisplay",
            "CategoryDynamicDropDownLevel",
            "DepartmentDynamicDropDownLevel",
            "RandomProductShow",
            "ProductBestSellingShow",
            "RecentlyViewedProductDisplay",
            "RecentlyViewedProductShow",
            "WidgetAddThisIsEnabled",
            "WidgetLikeBoxIsEnabled",
            "WidgetLivePersonIsEnabled",
            "RestrictAccessToShop",
            "PriceRequireLogin",
            "SaleTaxExempt",
            "CheckoutMode",
            "EnableBundlePromo",
            "EnableRMA",
            "EnableReviewPerCulture",
            "CategoryShowProductList",
            "DepartmentShowProductList",
            "SearchMode",
            "MobileView",
            "FacetedSearchEnabled",
            "PriceNavigationStep",
            "MaximunInterval",
            "EnableManufacturer",
            "ManufacturerMenuType",
            "EnableAddToCartNotification",
            "EnableQuickView",
            "AdvancedSearchMode",

            // Store Layout config
            "StoreBannerModuleDisplay",
            "StoreBannerEffectMode",
            "StoreBannerSlideSpeed",
            "StoreBannerEffectPeriod",
            "CategoryListModuleDisplay",
            "DepartMentListModuleDisplay",
            "DepartmentHeaderMenuDisplay",
            "ManufacturerHeaderMenuDisplay",
            "TodaySpecialModuleDisplay",
            "ProductSpecialEffectMode",
            "ProductSpecialTransitionSpeed",
            "ProductSpecialEffectWaitTime",
            "CurrencyModuleDisplay",
            "NewsletterModuleDisplay",
            "SpecialOfferModuleDisplay",
            "SpecialOfferImage",
            "PaymentLogoModuleDisplay",
            "PaymentLogoImage",
            "SearchModuleDisplay",
            "MiniCartModuleDisplay",
            "CouponModuleDisplay",
            "FeaturedMerchantModuleDisplay",
            "FeaturedMerchantCount",
            "FeaturedMerchantImage1",
            "FeaturedMerchantImage2",
            "FeaturedMerchantImage3",
            "FeaturedMerchantUrl1",
            "FeaturedMerchantUrl2",
            "FeaturedMerchantUrl3",
            "FreeShippingModuleDisplay",
            "FreeShippingImage",
            "SecureModuleDisplay",
            "SecureImage",
            "BestsellersModuleDisplay",
            "FeaturedProductModuleDisplay",
            "NewsModuleDisplay",
            "WishListEnabled",
            "TellAFriendEnabled",
            "GiftRegistryModuleDisplay",
            "DefaultDisplayCurrencyCode",
            "CompareListEnabled",
            "CompareProductShow",

            // Business Profile
            "CompanyZip",
            "CompanyPhone",
            "CompanyFax",
            "CompanyEmail",
            
            // Logo
            "LogoImage",
            
            // Email Setting
            "SmtpAuthenRequired",
            "SmtpUserName",
            "SmtpPassword",
            "SmtpServer",
            "SmtpPort",
            "SmtpSslRequired",

            //Content Management
            "TopContentMenu",            "LeftContentMenu",            "RightContentMenu",
            "TopContentMenuType",        "LeftContentMenuType",        "RightContentMenuType",

            //Store Configuration
            "DefaultWebsiteLanguage",
            "LanguageKeywordBaseCulture",
            "EnableSSL",
            "StoreDefaultCountry",
            "BundlePromotionShow",
            "BundlePromotionColumn",
            "BundlePromotionDisplay",

            //Google Sitemaps
            "GoogleSitemapsIsIncludesCategories",        
            "GoogleSitemapsIsIncludesProducts",
            "GoogleSitemapsIsIncludesContents",
            "GoogleSitemapsChangeFrequency",
            "GoogleSitemapsURLPriority",

            //Facebook
            "FacebookLoginEnabled",
            "FacebookAPIKey",
            "FacebookAppSecret",
            "FacebookAccessToken",

            //Blog
            "BlogEnabled",
            "BlogListItemsPerPage",
            "BlogTheme",
            "DefaultBlogListLayout",
            "DefaultBlogDetailsLayout",
            "FacebookCommentEnabled",
            "FacebookCommentAPIKey",
            "FacebookCommentNumberOfPosts",
            "BlogCommentEnabled",
            "BlogCommentsPerPage",
            "BlogCommentsSortOrder",

            //Point System
            "PointSystemEnabled",
            "RewardPoints",
            "PointValue",

            //New Arrival
            "EnableNewArrivalProduct",
            "ProductNewArrivalNumber",
            "MaximumDisplayProductNewArrival"
        };

        private static String[] _configsMultiCultureList = new String[] { 
            // Business Profile
            "CompanyName",
            "Title",
            "CompanyAddress",
            "CompanyCity",
            "CompanyState",
            "CompanyCountry",
        
            //Store Configuration
            "SiteName",
            "SiteDescription",
            "Sitekeyword",
            "BlogPageTitle",

            //SEO Configuration
            "DefaultProductPageTitle",
            "DefaultProductMetaKeyword",
            "DefaultProductMetaDescription",
            "DefaultCategoryPageTitle",
            "DefaultCategoryMetaKeyword",
            "DefaultCategoryMetaDescription",
            "DefaultDepartmentPageTitle",
            "DefaultDepartmentMetaKeyword",
            "DefaultDepartmentMetaDescription"
        };

        private string _storeID = "0";
        private string _storeName = String.Empty;
        private string _urlName = String.Empty;
        private bool _isEnabled = true;

        private bool IsExistConfigValue( Configuration config, string storeID )
        {
            foreach (ConfigurationValue value in config.Values)
            {
                if (value.StoreID == storeID)
                    return true;
            }
            return false;
        }

        private void CreateStoreConfigSingleCulture( string storeID, String[] configSingleList )
        {
            Store store = DataAccessContext.StoreRepository.GetOne( storeID );

            foreach (string configName in configSingleList)
            {
                if (!IsExistConfigValue( DataAccessContext.Configurations[configName], storeID ))
                {
                    DataAccessContext.ConfigurationRepository.UpdateValue(
                        DataAccessContext.Configurations[configName],
                        DataAccessContext.Configurations.GetValue( configName, Store.Null ),
                        store );
                }
            }
        }

        private void CreateStoreConfigMultiCulture( string storeID, String[] configsMultiList )
        {
            Store store = DataAccessContext.StoreRepository.GetOne( storeID );

            foreach (string configName in configsMultiList)
            {
                Configuration config = DataAccessContext.Configurations[configName];
                IList<ConfigurationValue> values = config.Values.Clone();

                if (!IsExistConfigValue( config, storeID ))
                {
                    foreach (ConfigurationValue value in values)
                    {
                        if (value.StoreID == Store.Null.StoreID)
                        {
                            DataAccessContext.ConfigurationRepository.UpdateValue(
                                DataAccessContext.CultureRepository.GetOne( value.CultureID ),
                                config,
                                value.ItemValue,
                                store );
                        }
                    }
                }
            }
        }

        private void SetUpStoreInformation( string storeID )
        {
            Culture culture = DataAccessContext.CultureRepository.GetOne(
            DataAccessContext.CultureRepository.GetIDByName( SystemConst.USCultureName ) );

            SetUpContentManagement( culture, storeID );
            SetupEmailTemplate( culture, storeID );
        }

        private void SetUpContentManagement( Culture culture, string storeID )
        {
            IList<ContentMenuItem> newContentMenuItemList = DataAccessContext.ContentMenuItemRepository.GetByStoreID(
                culture, storeID, "ContentMenuItemID", BoolFilter.ShowAll );

            if (newContentMenuItemList.Count == 0)
            {
                //Create Root Top Menu
                CreateContentMenuItem( storeID, "Top", "Root Top Menu", "Topmenu", "TopContentMenu" );

                //Create Root Left Menu
                CreateContentMenuItem( storeID, "Left", "Root Left Menu", "Leftmenu", "LeftContentMenu" );

                //Create Root Right Menu
                CreateContentMenuItem( storeID, "Right", "Root Right Menu", "Rightmenu", "RightContentMenu" );
            }

            DataAccessContext.ClearConfigurationCache();
        }

        private void SetupEmailTemplate( Culture culture, string storeID )
        {
            IList<EmailTemplateDetail> emailDetailList = DataAccessContext.EmailTemplateDetailRepository.GetAll( culture, "EmailTemplateDetailID", storeID );

            if (emailDetailList.Count == 0)
            {
                IList<EmailTemplateDetail> emailList = DataAccessContext.EmailTemplateDetailRepository.GetAll(
                    culture, "EmailTemplateDetailID", Store.RegularStoreID );

                foreach (EmailTemplateDetail email in emailList)
                {
                    IList<ILocale> emailLocaleList =
                        DataAccessContext.EmailTemplateDetailRepository.GetLocales( email.EmailTemplateDetailID );

                    email.EmailTemplateDetailID = "0";
                    email.StoreID = storeID;

                    foreach (ILocale locale in emailLocaleList)
                    {
                        EmailTemplateDetailLocale emailLocale = (EmailTemplateDetailLocale) locale;
                        emailLocale.EmailTemplateDetailID = "0";

                        email.Locales.Add( emailLocale );
                    }

                    DataAccessContext.EmailTemplateDetailRepository.Save( email );
                }

                DataAccessContext.ClearConfigurationCache();
            }
        }

        private void CreateContentMenuItem( string storeID, string name, string description, string urlName, string configName )
        {
            Store store = DataAccessContext.StoreRepository.GetOne( storeID );
            Culture culture = DataAccessContext.CultureRepository.GetOne(
                DataAccessContext.CultureRepository.GetIDByName( SystemConst.USCultureName ) );

            ContentMenuItem newItem = new ContentMenuItem( culture );
            newItem.Name = name;
            newItem.MenuPosition = ContentMenuItem.MenuPositionType.Root;
            newItem.Description = description;
            newItem.ContentMenuID = ContentMenuItem.RootMenuItemID;
            newItem.ContentID = "0";
            newItem.IsEnabled = false;

            ContentMenu newContentMenu = new ContentMenu();
            newContentMenu.IsEnabled = newItem.IsEnabled;
            newContentMenu = DataAccessContext.ContentMenuRepository.Save( newContentMenu );
            newItem.ReferringMenuID = newContentMenu.ContentMenuID;

            newItem.SortOrder = 1;
            newItem.UrlName = urlName;
            newItem.StoreID = storeID;

            DataAccessContext.ContentMenuItemRepository.Save( newItem );

            DataAccessContext.ConfigurationRepository.UpdateValue(
                    DataAccessContext.Configurations[configName],
                    newItem.ReferringMenuID,
                    store );

        }

        #endregion


        #region Public Properties

        public static Store Null
        {
            get { return NullStore.Instance; }
        }

        public virtual bool IsNull
        {
            get { return false; }
        }

        public const string RegularStoreID = "1";

        public string StoreID { get { return _storeID; } set { _storeID = value; } }
        public string StoreName { get { return _storeName; } set { _storeName = value; } }
        public string UrlName { get { return _urlName; } set { _urlName = value; } }
        public bool IsEnabled { get { return _isEnabled; } set { _isEnabled = value; } }


        #endregion

        #region Public Constructors

        public Store()
        {

        }

        #endregion

        #region ICloneable Members

        public object Clone()
        {
            Store newObject = (Store) MemberwiseClone();
            return newObject;
        }

        #endregion

        #region Public Method


        public void CreateStoreConfigCollection(
            string storeID, String[] configSingleList, String[] configMultiList )
        {
            ConfigurationCollection storeConfigCollection = new ConfigurationCollection();

            if (configSingleList.Length > 0)
                CreateStoreConfigSingleCulture( storeID, configSingleList );

            if (configMultiList.Length > 0)
                CreateStoreConfigMultiCulture( storeID, configMultiList );

            DataAccessContext.ClearConfigurationCache();
            SetUpStoreInformation( storeID );

        }

        public void CreateStoreConfigCollection( string storeID )
        {
            CreateStoreConfigCollection(
               storeID, _configsSingleCultureList, _configsMultiCultureList );
        }

        public static bool CanUseMultiStoreConfig( string configName )
        {
            foreach (string name in _configsSingleCultureList)
            {
                if (String.Compare( name, configName, true ) == 0)
                    return true;
            }

            foreach (string name in _configsMultiCultureList)
            {
                if (String.Compare( name, configName, true ) == 0)
                    return true;
            }

            return false;
        }

        #endregion
    }
}
