using System;
using System.Collections.Generic;
using System.Text;

using Vevo.Domain.Configurations;
using Vevo.Domain.Discounts;
using System.Collections;
using Vevo.Domain.Base;
using Vevo.Domain.Contents;
using Vevo.Base.Domain;

namespace Vevo.Domain.Users
{
    public class Customer : ICloneable
    {
        #region Private

        private string _customerID = "0";
        private DateTime _registerDate = DateTime.Now;
        private string _userName = String.Empty;
        private Address _billingAddress = new Address();

        private string _email = String.Empty;
        private bool _useBillingAsShipping = true;
        private string _merchantNotes = String.Empty;
        private bool _isWholesale = false;
        private int _wholesaleLevel = 0;
        private string _affiliateCode = String.Empty;
        private DateTime _affiliateExpireDate = DateTime.Now;
        private bool _isTaxExempt = false;
        private bool _isEnabled = true;
        private string _qBListID = String.Empty;
        private bool _isQBExported = false;
        private string _productIDsCompareList = String.Empty;
        private string _taxExemptID = String.Empty;
        private string _taxExemptCountry = String.Empty;
        private string _taxExemptState = String.Empty;
        private string _fbUserID = String.Empty;
        private IEntityList<ShippingAddress> _shippingAddress;

        #endregion


        #region Public Constructors

        public Customer()
        {
        }

        #endregion


        #region Public Properties

        public static Customer Null
        {
            get { return NullCustomer.Instance; }
        }

        public virtual bool IsNull
        {
            get { return false; }
        }

        public string CustomerID { get { return _customerID; } set { _customerID = value; } }
        public DateTime RegisterDate { get { return _registerDate; } set { _registerDate = value; } }
        public string UserName { get { return _userName; } set { _userName = value; } }
        public Address BillingAddress { get { return _billingAddress; } set { _billingAddress = value; } }
        public string Email { get { return _email; } set { _email = value; } }
        public bool UseBillingAsShipping { get { return _useBillingAsShipping; } set { _useBillingAsShipping = value; } }
        public string MerchantNotes { get { return _merchantNotes; } set { _merchantNotes = value; } }
        public bool IsWholesale { get { return _isWholesale; } set { _isWholesale = value; } }
        public int WholesaleLevel { get { return _wholesaleLevel; } set { _wholesaleLevel = value; } }
        public string AffiliateCode { get { return _affiliateCode; } set { _affiliateCode = value; } }
        public DateTime AffiliateExpireDate { get { return _affiliateExpireDate; } set { _affiliateExpireDate = value; } }
        public bool IsTaxExempt { get { return _isTaxExempt; } set { _isTaxExempt = value; } }
        public bool IsEnabled { get { return _isEnabled; } set { _isEnabled = value; } }
        public string QBListID { get { return _qBListID; } set { _qBListID = value; } }
        public bool IsQBExported { get { return _isQBExported; } set { _isQBExported = value; } }
        public string ProductIDsCompareList { get { return _productIDsCompareList; } set { _productIDsCompareList = value; } }
        public string TaxExemptID { get { return _taxExemptID; } set { _taxExemptID = value; } }
        public string TaxExemptCountry { get { return _taxExemptCountry; } set { _taxExemptCountry = value; } }
        public string TaxExemptState { get { return _taxExemptState; } set { _taxExemptState = value; } }
        public string FBUserID { get { return _fbUserID; } set { _fbUserID = value; } }

        public ShippingAddress ShippingAddress
        {
            get { return ShippingAddresses[0]; }
        }

        #endregion


        #region IClonable Members

        // Copy the whole aggregate (including locale)
        public object Clone()
        {
            Customer newObject = (Customer) MemberwiseClone();
            return newObject;
        }

        #endregion

        #region Custom Properties

        public IEntityList<ShippingAddress> ShippingAddresses
        {
            get
            {
                if (_shippingAddress == null)
                    _shippingAddress = new EntityList<ShippingAddress>(
                        DataAccessContext.CustomerRepository.GetShippingAddressByCustomerID( CustomerID ) );

                return _shippingAddress;
            }
        }

        #endregion

        #region Public Method

        public static bool ValidCustomerUserNames( string[] customerUserNames )
        {
            bool result = true;
            if (customerUserNames != null)
            {
                for (int i = 0; i < customerUserNames.Length; i++)
                {
                    if (DataAccessContext.CustomerRepository.GetIDFromUserName( customerUserNames[i].Trim() ) == "0")
                    {
                        result = false;
                    }
                }
            }

            return result;
        }

        public WholesaleStatus GetWholesaleStatus()
        {
            return new WholesaleStatus( DataAccessContext.Configurations, WholesaleLevel );
        }

        public WholesaleStatus GetWholesaleStatus( ConfigurationCollection configurations )
        {
            return new WholesaleStatus( configurations, WholesaleLevel );
        }

        public ArrayList GetProductIDsCompare()
        {
            if (!String.IsNullOrEmpty( ProductIDsCompareList ))
            {
                ArrayList list = new ArrayList( ProductIDsCompareList.Split( ',' ) );
                return list;
            }
            else
                return new ArrayList();
        }

        public void SetProductIDsCompare( ArrayList list )
        {
            ProductIDsCompareList = String.Join( ",", (string[]) list.ToArray( Type.GetType( "System.String" ) ) );
        }

        public ShippingAddress GetShippingAddress( string shippingAddressID )
        {
            foreach (ShippingAddress address in ShippingAddresses)
            {
                if (address.ShippingAddressID == shippingAddressID)
                    return address;
            }

            return null;
        }

        public void SetShippingAddress( ShippingAddress shippingAddress )
        {
            int deleteIndex = -1;

            for (int i = 0; i < ShippingAddresses.Count; i++)
            {
                if (ShippingAddresses[i].ShippingAddressID == shippingAddress.ShippingAddressID)
                {
                    deleteIndex = i;

                }
            }

            ShippingAddresses.RemoveAt( deleteIndex );
            ShippingAddresses.Add( shippingAddress );
        }

        #endregion
    }
}
